/***********************************************************************
 *    DecoEmojiList
 *
 * [File]
 *     DecoEmojiList.java
 *
 *     Copyright (C) 2014 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     DecoEmojiList Main Activity Class.
 ************************************************************************/
package com.emojisampleapplication;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.text.Spannable;
import android.text.SpannableString;
import android.util.Log;
import android.view.GestureDetector;
import android.view.GestureDetector.OnGestureListener;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.emojisampleapplication.constants.Const;

import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiCategoryInfo;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiContract;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.IDecoEmojiConstant;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.IDecoEmojiManager;
import jp.co.omronsoft.android.emoji.EmojiAssist;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

/**
 * 一覧表示制御アプリ メインアクティビティクラス.
 */
public class DecoEmojiList extends Activity implements Runnable {

    /** For Debug Class Name. */
    private static final String CLASS_NAME = "DecoEmojiList#";

    private static final int INPUT_TYPE_SINGLE = 1;

    /** 一覧に表示される絵文字のサイズ(sp) */
    private static final float DECOEMOJI_SIZE = 20.0f;

    /** 絵文字タイプを取得するための文字列 */
    private static final String EMOJI_TYPE = "emojiType";

    /** 絵文字Dリソース読み込みリトライ */
    private static final int RETRY_TIME = 5000;
    private static final int MAX_RETRY_COUNT = 6;

    /** 絵文字Dリソースチェック */
    private static final int NO_PICT_D_DATA_FLAG = 1;

    /** Candidates of emoji_unicode_faces */
    private static final String[] UNICODE_EMOJI_FACES_TABLE = { "\ud83d\ude0a",
            "\ud83d\ude04", "\ud83d\ude09", "\ud83d\ude06", "\ud83d\ude0b",
            "\ud83d\ude01", "\ud83d\ude02", "\ud83d\ude03", "\ud83d\ude0f",
            "\ud83d\ude0d", "\ud83d\ude18", "\ud83d\ude1a", "\ud83d\ude0c",
            "\ud83d\ude12", "\ud83d\ude05", "\ud83d\ude13", "\ud83d\ude25",
            "\ud83d\ude30", "\ud83d\ude1e", "\ud83d\ude14", "\ud83d\ude16",
            "\ud83d\ude29", "\ud83d\ude35", "\ud83d\ude23", "\ud83d\ude32",
            "\ud83d\ude28", "\ud83d\ude31", "\ud83d\ude2b", "\ud83d\ude2d",
            "\ud83d\ude22", "\ud83d\ude20", "\ud83d\ude21", "\ud83d\ude1c",
            "\ud83d\ude1d", "\ud83d\ude2a", "\ud83d\ude24", "\ud83d\ude33",
            "\ud83d\ude37", "\u263a", "\ud83d\ude3a", "\ud83d\ude38",
            "\ud83d\ude39", "\ud83d\ude3d", "\ud83d\ude3b", "\ud83d\ude3f",
            "\ud83d\ude3e", "\ud83d\ude3c", "\ud83d\ude40", "\ud83d\ude45",
            "\ud83d\ude46", "\ud83d\ude47", "\ud83d\ude4b", "\ud83d\ude4c",
            "\ud83d\ude4d", "\ud83d\ude4e", "\ud83d\ude4f", "\ud83d\ude48",
            "\ud83d\ude4a", "\ud83d\ude49", "\ud83d\udc66", "\ud83d\udc67",
            "\ud83d\udc68", "\ud83d\udc69", "\ud83d\udc6f", "\ud83d\udc70",
            "\ud83d\udc78", "\ud83d\udc71", "\ud83d\udc72", "\ud83d\udc73",
            "\ud83d\udc74", "\ud83d\udc75", "\ud83d\udc76", "\ud83d\udc6a",
            "\ud83d\udc6b", "\ud83d\udc8f", "\ud83d\udc91", "\ud83d\udc6e",
            "\ud83d\udc77", "\ud83d\udc82", "\ud83d\udc81", "\ud83d\udc7b",
            "\ud83d\udc7c", "\ud83d\udc7f", "\ud83d\udc7d", "\ud83d\udc7e",
            "\ud83d\udc80", "\ud83d\udc79", "\ud83d\udc7a", "\ud83d\udc64" };

    /** Candidates of emoji_unicode_human */
    private static final String[] UNICODE_EMOJI_HUMAN_TABLE = { "\u2764",
            "\ud83d\udc93", "\ud83d\udc94", "\ud83d\udc95", "\ud83d\udc96",
            "\ud83d\udc97", "\ud83d\udc98", "\ud83d\udc99", "\ud83d\udc9a",
            "\ud83d\udc9b", "\ud83d\udc9c", "\ud83d\udc9d", "\ud83d\udc9e",
            "\ud83d\udc9f", "\u2728", "\u2734", "\u2b50", "\ud83c\udf1f",
            "\ud83c\udf20", "\u2747", "\ud83c\udfb5", "\ud83c\udfb6",
            "\ud83d\udc90", "\ud83c\udf89", "\ud83c\udf8a", "\ud83d\udcae",
            "\ud83d\udcaf", "\ud83d\udca1", "\ud83d\udcab", "\ud83d\udca6",
            "\ud83d\udca7", "\ud83d\udca8", "\ud83d\udca4", "\ud83d\udca2",
            "\ud83d\udca5", "\ud83d\udca3", "\ud83d\udd25", "\u26a0",
            "\ud83c\udd98", "\ud83d\udcb8", "\ud83d\udca9", "\u270a", "\u270c",
            "\u270b", "\ud83d\udc4a", "\ud83d\udc4d", "\u261d", "\ud83d\udc46",
            "\ud83d\udc47", "\ud83d\udc48", "\ud83d\udc49", "\ud83d\udc4b",
            "\ud83d\udc4f", "\ud83d\udc4c", "\ud83d\udc4e", "\ud83d\udc50",
            "\ud83d\udcaa", "\ud83d\udc40", "\ud83d\udc42", "\ud83d\udc43",
            "\ud83d\udc8b", "\ud83d\udc44", "\ud83d\udc45", "\ud83d\udc63",
            "\u2757", "\u2755", "\u203c", "\u2049", "\u2753", "\u2754" };

    /** Candidates of emoji_unicode_critters */
    private static final String[] UNICODE_EMOJI_CRITTERS_TABLE = {
            "\ud83d\udc36", "\ud83d\udc31", "\ud83d\udc3e", "\ud83d\udc30",
            "\ud83d\udc3b", "\ud83d\udc3c", "\ud83d\udc35", "\ud83d\udc12",
            "\ud83d\udc39", "\ud83d\udc34", "\ud83d\udc18", "\ud83d\udc2f",
            "\ud83d\udc3a", "\ud83d\udc0d", "\ud83d\udc2d", "\ud83d\udc17",
            "\ud83d\udc38", "\ud83d\udc2b", "\ud83d\udc29", "\ud83d\udc28",
            "\ud83d\udc11", "\ud83d\udc37", "\ud83d\udc3d", "\ud83d\udc2e",
            "\ud83d\udc14", "\ud83d\udc24", "\ud83d\udc25", "\ud83d\udc23",
            "\ud83d\udc26", "\ud83d\udc27", "\ud83d\udc1f", "\ud83d\udc2c",
            "\ud83d\udc33", "\ud83d\udc19", "\ud83d\udc20", "\ud83d\udc21",
            "\ud83d\udc1a", "\ud83d\udc22", "\ud83d\udc32", "\ud83d\udc1c",
            "\ud83d\udc1e", "\ud83d\udc0c", "\ud83d\udc1b", "\ud83d\udc1d" };

    /** Candidates of emoji_unicode_foods */
    private static final String[] UNICODE_EMOJI_FOODS_TABLE = { "\ud83c\udf5a",
            "\ud83c\udf59", "\ud83c\udf5e", "\ud83c\udf54", "\ud83c\udf5f",
            "\ud83c\udf5c", "\ud83c\udf64", "\ud83c\udf5d", "\ud83c\udf5b",
            "\ud83c\udf62", "\ud83c\udf63", "\ud83c\udf71", "\ud83c\udf72",
            "\ud83c\udf65", "\ud83c\udf60", "\ud83c\udf55", "\ud83c\udf56",
            "\ud83c\udf57", "\ud83c\udf82", "\ud83c\udf70", "\ud83c\udf68",
            "\ud83c\udf67", "\ud83c\udf66", "\ud83c\udf69", "\ud83c\udf6a",
            "\ud83c\udf6b", "\ud83c\udf6c", "\ud83c\udf6d", "\ud83c\udf6e",
            "\ud83c\udf6f", "\ud83c\udf61", "\ud83c\udf58", "\ud83c\udf73",
            "\ud83d\udd2a", "\ud83c\udf74", "\u2615", "\ud83c\udfee",
            "\ud83c\udf78", "\ud83c\udf7a", "\ud83c\udf75", "\ud83c\udf76",
            "\ud83c\udf77", "\ud83c\udf7b", "\ud83c\udf79", "\ud83c\udf52",
            "\ud83c\udf4c", "\ud83c\udf4e", "\ud83c\udf4f", "\ud83c\udf4a",
            "\ud83c\udf53", "\ud83c\udf49", "\ud83c\udf45", "\ud83c\udf46",
            "\ud83c\udf48", "\ud83c\udf4d", "\ud83c\udf47", "\ud83c\udf51",
            "\ud83c\udf3d", "\ud83c\udf44", "\ud83c\udf30" };

    /** Candidates of emoji_unicode_nature */
    private static final String[] UNICODE_EMOJI_NATURE_TABLE = { "\u2600",
            "\u2601", "\u26c5", "\u2614", "\ud83c\udf08", "\u26c4", "\u2744",
            "\u26a1", "\ud83c\udf00", "\ud83c\udf01", "\ud83c\udf04",
            "\ud83c\udf05", "\ud83c\udf06", "\ud83c\udf07", "\ud83c\udf03",
            "\ud83c\udf09", "\ud83c\udf11", "\ud83c\udf14", "\ud83c\udf13",
            "\ud83c\udf19", "\ud83c\udf15", "\ud83c\udf1b", "\ud83c\udf0f",
            "\ud83c\udf0a", "\ud83c\udf0b", "\ud83c\udf40", "\ud83c\udf37",
            "\ud83c\udf31", "\ud83c\udf3f", "\ud83c\udf38", "\ud83c\udf39",
            "\ud83c\udf3c", "\ud83c\udf3a", "\ud83c\udf3b", "\ud83c\udf34",
            "\ud83c\udf35", "\ud83c\udf43", "\ud83c\udf42", "\ud83c\udf41",
            "\ud83c\udf3e", "\ud83c\udf8d", "\ud83c\udf8e", "\ud83c\udf93",
            "\ud83c\udf92", "\ud83c\udf8f", "\ud83c\udf8b", "\ud83c\udf0c",
            "\ud83c\udf90", "\ud83c\udf87", "\ud83c\udf86", "\ud83c\udf91",
            "\ud83c\udf83", "\ud83c\udf84", "\ud83c\udf85" };

    /** Candidates of emoji_unicode_activities */
    private static final String[] UNICODE_EMOJI_ACTIVITIES_TABLE = {
            "\ud83d\udc55", "\ud83d\udc5a", "\ud83c\udfbd", "\ud83d\udc54",
            "\ud83d\udc56", "\ud83d\udc57", "\ud83d\udc58", "\ud83d\udc59",
            "\ud83d\udc5e", "\ud83d\udc5f", "\ud83d\udc60", "\ud83d\udc61",
            "\ud83d\udc62", "\ud83d\udc5c", "\ud83d\udcbc", "\ud83d\udc5d",
            "\ud83c\udf02", "\ud83d\udc52", "\ud83c\udfa9", "\ud83d\udc51",
            "\ud83d\udd31", "\ud83d\udc53", "\ud83d\udc8d", "\ud83d\udc8e",
            "\ud83c\udf80", "\ud83d\udc84", "\ud83d\udc85", "\ud83d\udc86",
            "\ud83d\udc87", "\ud83d\udc88", "\u26be", "\u26bd", "\ud83c\udfc0",
            "\u26f3", "\ud83c\udfbe", "\ud83c\udfc8", "\ud83d\udeb6",
            "\ud83c\udfc3", "\ud83c\udfc4", "\ud83c\udfca", "\ud83c\udfc2",
            "\ud83c\udfbf", "\ud83d\udc83", "\ud83d\udc0e", "\ud83c\udfc6",
            "\ud83c\udfc1", "\ud83c\udfa3", "\u2668", "\u26fa", "\ud83c\udfa0",
            "\ud83c\udfa1", "\ud83c\udfa2", "\ud83c\udfaa", "\ud83c\udfac",
            "\ud83c\udfad", "\ud83c\udfae", "\ud83d\udd2e", "\ud83c\udfb1",
            "\ud83c\udfb3", "\ud83c\udfb2", "\ud83c\udfaf", "\ud83c\udfb0",
            "\ud83c\udc04", "\ud83c\udfb4", "\ud83c\udccf", "\ud83c\udfb7",
            "\ud83c\udfb8", "\ud83c\udfb9", "\ud83c\udfba", "\ud83c\udfbb",
            "\ud83c\udfbc" };

    /** Candidates of emoji_unicode_artifacts */
    private static final String[] UNICODE_EMOJI_ARTIFACTS_TABLE = {
            "\ud83d\udeb2", "\ud83d\ude97", "\ud83d\ude99", "\ud83d\ude95",
            "\ud83d\ude9a", "\ud83d\ude92", "\ud83d\ude91", "\ud83d\ude93",
            "\ud83d\udea8", "\ud83d\ude8c", "\ud83d\ude8f", "\ud83d\ude83",
            "\ud83d\ude89", "\ud83d\ude87", "\u24c2", "\ud83d\ude84",
            "\ud83d\ude85", "\ud83d\udcba", "\u2708", "\ud83d\ude80", "\u2693",
            "\u26f5", "\ud83d\udea2", "\ud83d\udea4", "\ud83c\udfe0",
            "\ud83c\udfe1", "\ud83c\udfe2", "\ud83c\udfeb", "\ud83c\udfe3",
            "\ud83c\udfe5", "\ud83c\udfe6", "\ud83c\udfe7", "\u26fd",
            "\ud83c\udfea", "\ud83c\udfe8", "\ud83c\udfe9", "\u26f2", "\u26ea",
            "\ud83d\udc92", "\ud83c\udfec", "\ud83c\udfef", "\ud83c\udff0",
            "\ud83c\udfed", "\ud83d\udce1", "\ud83d\udd30", "\u26d4",
            "\ud83c\udd7f", "\ud83d\udea5", "\ud83d\udea7", "\ud83d\udea9",
            "\ud83d\udebb", "\ud83d\udebd", "\ud83d\udebe", "\ud83d\udec0",
            "\ud83d\uddfe", "\ud83d\uddfb", "\ud83d\uddfc", "\ud83d\uddfd",
            "\ud83d\uddff" };

    /** Candidates of emoji_unicode_tools */
    private static final String[] UNICODE_EMOJI_TOOLS_TABLE = { "\u260e",
            "\ud83d\udcde", "\ud83d\udcf1", "\ud83d\udcbb", "\ud83d\udce0",
            "\ud83d\udcdf", "\ud83d\udcfa", "\ud83d\udcfb", "\ud83d\udcf0",
            "\ud83d\udcf7", "\ud83d\udcf9", "\ud83c\udfa4", "\ud83c\udfa7",
            "\ud83c\udfa5", "\ud83c\udfa6", "\ud83c\udfa8", "\ud83c\udfab",
            "\ud83d\udcbf", "\ud83d\udcc0", "\ud83d\udcfc", "\ud83d\udcbd",
            "\ud83d\udcbe", "\u231a", "\u23f0", "\u23f3", "\u231b",
            "\ud83d\udcdb", "\ud83c\udf88", "\ud83d\udc89", "\ud83d\udc8a",
            "\ud83d\udc5b", "\ud83d\udcb3", "\ud83d\udcb0", "\u270f", "\u2712",
            "\ud83d\udcce", "\u2702", "\ud83d\udccd", "\ud83d\udccc",
            "\ud83d\udccf", "\ud83d\udcd0", "\ud83d\udcd7", "\ud83d\udcd8",
            "\ud83d\udcd9", "\ud83d\udcd5", "\ud83d\udcda", "\ud83d\udcd6",
            "\ud83d\udcc1", "\ud83d\udcc2", "\ud83d\udcd1", "\ud83d\udd16",
            "\ud83d\udcc3", "\ud83d\udcc4", "\ud83d\udcd3", "\ud83d\udcd4",
            "\ud83d\udcd2", "\ud83d\udccb", "\ud83d\udcdd", "\ud83d\udcc7",
            "\ud83d\udcc5", "\ud83d\udcc6", "\ud83d\udcb9", "\ud83d\udcca",
            "\ud83d\udcc8", "\ud83d\udcc9", "\ud83d\udd27", "\ud83d\udd28",
            "\ud83d\udd29", "\ud83d\udeaa", "\ud83d\udd2b", "\ud83d\udd26",
            "\ud83d\udd0b", "\ud83d\udd0c", "\ud83d\udd14", "\ud83d\udce2",
            "\ud83d\udce3", "\ud83d\udd0a", "\ud83d\udd0d", "\ud83d\udd0e",
            "\ud83d\udcea", "\ud83d\udceb", "\ud83d\udcee", "\ud83d\udce6",
            "\ud83c\udf81", "\ud83d\udd11", "\ud83d\udd12", "\ud83d\udd13",
            "\ud83d\udd10", "\ud83d\udd0f" };

    /** Candidates of emoji_unicode_symbols */
    private static final String[] UNICODE_EMOJI_SYMBOLS_TABLE = {
            "\uD83D\uDCF2", "\uD83D\uDCF3", "\uD83D\uDCF4", "\uD83D\uDCF6",
            "\u2709", "\uD83D\uDCE7", "\uD83D\uDCE9", "\uD83D\uDCE8",
            "\uD83D\uDC8C", "\uD83D\uDCAC", "\uD83D\uDCE4", "\uD83D\uDCE5",
            "\uD83C\uDD97", "\uD83C\uDD96", "\uD83C\uDD9A", "\uD83C\uDD95",
            "\uD83C\uDD99", "\uD83C\uDD93", "\uD83C\uDD94", "\uD83C\uDD91",
            "\uD83D\uDD1D", "\uD83C\uDD92", "\uD83C\uDE36", "\uD83C\uDE1A",
            "\uD83C\uDE37", "\uD83C\uDE38", "\uD83C\uDE39", "\uD83C\uDE02",
            "\uD83C\uDE2F", "\uD83C\uDE3A", "\uD83C\uDE32", "\uD83C\uDE33",
            "\uD83C\uDE34", "\uD83C\uDE35", "\uD83C\uDE01", "\u3297",
            "\uD83C\uDE51", "\uD83C\uDE50", "\u3299", "\uD83D\uDEAB",
            "\uD83D\uDD1E", "\uD83D\uDEAD", "\uD83D\uDEAC", "\uD83D\uDEB9",
            "\uD83D\uDEBA", "\uD83D\uDEBC", "\u2139", "\u267F", "\u267B",
            "\u00A9", "\u00AE", "\u2122", "\uD83D\uDCB4", "\uD83D\uDCB2",
            "\uD83D\uDCB5", "\uD83D\uDCB1", "\u27BF", "\u2795", "\u2796",
            "\u2716", "\u2797", "\uD83D\uDCA0", "\u2733", "\uD83D\uDD17",
            "\uD83D\uDCDC", "\u303D", "\u3030", "\u27B0", "\u2934", "\u2935",
            "\u2B06", "\u2B07", "\u27A1", "\u2B05", "\u2195", "\u2194",
            "\u2197", "\u2198", "\u2196", "\u2199", "\u21AA", "\u21A9",
            "\uD83D\uDD1B", "\uD83D\uDD1A", "\uD83D\uDD1C", "\uD83D\uDD19",
            "\uD83D\uDD03", "\u25B6", "\u25C0", "\u23E9", "\u23EA",
            "\uD83D\uDD3C", "\uD83D\uDD3D", "\u23EB", "\u23EC", "\uD83D\uDD3A",
            "\uD83D\uDD3B", "\uD83D\uDD50", "\uD83D\uDD51", "\uD83D\uDD52",
            "\uD83D\uDD53", "\uD83D\uDD54", "\uD83D\uDD55", "\uD83D\uDD56",
            "\uD83D\uDD57", "\uD83D\uDD58", "\uD83D\uDD59", "\uD83D\uDD5A",
            "\uD83D\uDD5B", "\u2648", "\u2649", "\u264A", "\u264B", "\u264C",
            "\u264D", "\u264E", "\u264F", "\u2650", "\u2651", "\u2652",
            "\u2653", "\u26CE", "\uD83D\uDD2F", "\uD83C\uDD70", "\uD83C\uDD71",
            "\uD83C\uDD8E", "\uD83C\uDD7E", "\uD83D\uDD20", "\uD83D\uDD21",
            "\uD83D\uDD22", "\uD83D\uDD23", "\uD83D\uDD24", "\u2665", "\u2660",
            "\u2666", "\u2663", "\u2B55", "\u274C", "\u274E", "\u2714",
            "\u2705", "\u2611", "\uD83D\uDD18", "\uD83D\uDD34", "\uD83D\uDD35",
            "\u26AA", "\u26AB", "\uD83D\uDD33", "\u2B1C", "\u2B1B", "\u25AB",
            "\u25AA", "\u25FD", "\u25FE", "\u25FB", "\u25FC", "\uD83D\uDD32",
            "\uD83D\uDD36", "\uD83D\uDD37", "\uD83D\uDD38", "\uD83D\uDD39",
            "\ud83c\udf8c" };

    /** Tables of Unicode emoji category */
    private static final String[][] UNICODE_EMOJI_TABLES = {
            UNICODE_EMOJI_FACES_TABLE, UNICODE_EMOJI_HUMAN_TABLE,
            UNICODE_EMOJI_CRITTERS_TABLE, UNICODE_EMOJI_FOODS_TABLE,
            UNICODE_EMOJI_NATURE_TABLE, UNICODE_EMOJI_ACTIVITIES_TABLE,
            UNICODE_EMOJI_ARTIFACTS_TABLE, UNICODE_EMOJI_TOOLS_TABLE,
            UNICODE_EMOJI_SYMBOLS_TABLE };

    /** DecoEmojiManagerから取得したカテゴリ情報一覧を表すフィールド. */
    private DecoEmojiCategoryInfo[] mCategoryInfoArray;

    /** 現在表示しているデコ絵文字の情報一覧を表すフィールド. */
    private List<EmojiAssist.DecoEmojiTextInfo> mCurrentDecoEmojiList;

    /** 現在表示しているカテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentCategoryDataIndex = 0;

    /** 現在表示しているUnicode絵文字カテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentUnicodeCategoryDataIndex = 0;

    /** 回転前のカテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentCategoryDataIndexTemp = 0;

    /** 回転前のUnicode絵文字カテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentUnicodeCategoryDataIndexTemp = 0;

    /** 現在表示しているカテゴリ情報のページ数を表すフィールド. */
    private int mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;

    /** 入力履歴テーブル操作用DAOクラスを表すフィールド. */
    private DecoEmojiListDao mInputHistoryDao;

    /** IMEに返却するリストを表すフィールド. */
    private ArrayList<DecoEmojiListResultInfo> mReturnValue = new ArrayList<DecoEmojiListResultInfo>();

    /** デコ絵文字一覧のスクロールビューを表すフィールド. */
    private ScrollView mBaseScroll;

    /** デコ絵文字一覧をスクロールさせるレイアウトを表すフィールド. */
    private LinearLayout mDecoEmojiListBaseLinear;

    /** デコ絵文字一覧の横を整形するレイアウトを表すフィールド. */
    private LinearLayout mDecoEmojiListInner;

    /** カテゴリ内ページ切替(上)ボタンを表すフィールド. */
    private ImageView mReadMoreButtonBack;

    /** カテゴリ内ページ切替(下)ボタンを表すフィールド. */
    private ImageView mReadMoreButtonNext;

    /** 確定ボタンを表すフィールド. */
    private ImageView mEnterButton;

    /** プレビューテキストを表すフィールド. */
    private TextView mPreviewText;

    /** デコ絵文字タブを表すフィールド. */
    private TextView mSwitchEmojiTabDeco;

    /** Unicode絵文字タブを表すフィールド. */
    private TextView mSwitchEmojiTabUnicode;

    /** カテゴリ名称を表すフィールド. */
    private Spinner mCategoryName;

    /** バックスペースボタンを表すフィールド. */
    private ImageView mBackSpaceButton;

    /** ボタン領域を表すフィールド. */
    private LinearLayout mBaseButtonsLinear;

    /** 入力履歴が存在するかを表すフィールド. */
    private boolean mIsExistInputHistory;

    /** ListViewManagerを表すフィールド. */
    private DecoEmojiListViewManager mListViewManager;

    /** DecoEmojiManagerバインド用オブジェクトを表すフィールド. */
    private IDecoEmojiManager mIDecoEmojiManagerIF = null;

    /** 現在のロケールが日本語かを表すフィールド. */
    private boolean mIsLanguageJapan = false;

    /** デコ／Unicode絵文字切替判定用文字列を表すフィールド. */
    private int mSwitchEmojiFlg;

    /** Unicode絵文字、デコレーション絵文字の表示個数切替用 */
    private int mCurrentInputType;

    /** 現在表示している絵文字を表すフィールド. */
    private int mCurrentEmojiValue;

    /** IMEの領域を表すフィールド. [0]：縦画面時の縦幅 [1]：横画面時の縦幅 */
    private int[] mCurrentImeHeight = new int[2];

    /** フリックイベント時のフリック方向を表すフィールド. */
    private int mFlickDirection = Const.FLICK_NONE;

    /** フリックイベント等を検知するジェスチャーを表すフィールド. */
    private GestureDetector mGestureDetector;

    /** フリックされたかを表すフィールド. */
    private boolean mIsFlick = false;

    /** ページスクロール時のスクロール開始地点(Y座標)を表すフィールド. */
    private int mScrollFrom = 0;

    /** ページスクロール時のスクロール終了地点(Y座標)を表すフィールド. */
    private int mScrollTo = 0;

    /** finish()を実行したかを表すフィールド. */
    private boolean isSendFinish = false;

    /** デコ絵文字描画用インスタンスを表すフィールド. */
    private EmojiAssist mEmojiAssist;

    /** Unicode絵文字カテゴリリスト */
    private List<UnicodeEmojiCategoryInfo> mUnicodeEmojiCategoryInfoList;

    /** Unicode絵文字入力履歴が存在するかを表すフィールド */
    private boolean mIsExistUnicodeInputHistory;

    private int mRetryCount = MAX_RETRY_COUNT;

    private int mNoPictDData = 0;

    private boolean mCategoryInfoCheck = false;

    /** 絵文字Dリソース読み込みリトライ用スレッド */
    private Thread retryThread;

    /** 絵文字種別 */
    private int mEmojiType = 0x0000;

    /** ページの絵文字表示情報 */
    private LinkedHashMap<Integer, PageInfo> mPageInfo = new LinkedHashMap<Integer, PageInfo>();

    /** 最大ページを表すフィールド */
    private int mMaxPage = 0;

    /** View's. */
    private ArrayList<TextView> mAddViewList;

    /** @see android.app.Activity#onCreate */
    @Override
    public void onCreate(Bundle b) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onCreate() start.");
        }

        super.onCreate(b);

        mAddViewList = new ArrayList<TextView>();
        mAddViewList.clear();

        Intent intentFromIme = (Intent) getIntent();

        // 入力履歴テーブル操作用DAOクラスを生成する
        mInputHistoryDao = new DecoEmojiListDao(getApplicationContext());
        // Unicode絵文字一覧を取得する
        loadUnicodeEmojiResources();

        // 絵文字描画用インスタンスを取得する
        mEmojiAssist = EmojiAssist.getInstance();
        mEmojiAssist.setPictureScale(true);

        // デコ／Unicode絵文字タブ切替のフラグを設定する
        mSwitchEmojiFlg = intentFromIme.getIntExtra(
                Const.EXTRA_SWITCH_EMOJI_FLG, Const.SWITCH_VALUE_BOTH);
        if (mSwitchEmojiFlg != Const.SWITCH_VALUE_DECO
                && mSwitchEmojiFlg != Const.SWITCH_VALUE_UNICODE) {
            // デコ／Unicode絵文字タブ
            mSwitchEmojiFlg = Const.SWITCH_VALUE_BOTH;
            mCurrentEmojiValue = Const.SWITCH_VALUE_DECO;
        } else {
            mCurrentEmojiValue = mSwitchEmojiFlg;
        }

        mEmojiType = intentFromIme.getIntExtra(EMOJI_TYPE, EmojiAssist.EMOJITYPE_PICTD_SQ20);

        // 起動元のIMEの縦幅を設定する
        mCurrentImeHeight[0] = intentFromIme.getIntExtra(
                Const.EXTRA_CURRENT_IME_HEIGHT_PORT, 0);
        mCurrentImeHeight[1] = intentFromIme.getIntExtra(
                Const.EXTRA_CURRENT_IME_HEIGHT_LAND, 0);
        mCurrentInputType = intentFromIme.getIntExtra(
                Const.EXTRA_CURRENT_INPUT_TYPE, 0);

        // ロケールを設定する
        mIsLanguageJapan = Locale.JAPAN.equals(Locale.getDefault());

        // 画面に表示する情報をコンテントに登録する
        mListViewManager = new DecoEmojiListViewManager();
        WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
        int width = wm.getDefaultDisplay().getWidth();
        int height = wm.getDefaultDisplay().getHeight();
        Resources res = getResources();
        LayoutInflater inflater = getLayoutInflater();
        ViewGroup viewGroup = mListViewManager.initView(res, inflater, width,
                height, mCurrentImeHeight, mCurrentInputType);
        setContentView(viewGroup);

        // 画面に配置するコンポーネントの設定を行う
        setComponentEvents();

        // デコ／Unicode絵文字タブの設定を行う
        setSwitchEmojiTabEvent();

        // カテゴリ名称を設定する
        setCategoryName();

        // サービス(DecoEmojiManager)をバインドする
        Intent intentBindSearvice = new Intent();
        intentBindSearvice.setClassName(IDecoEmojiConstant.PACKAGE_NAME, IDecoEmojiConstant.CLASS_NAME);
        bindService(intentBindSearvice, serviceConn, Context.BIND_AUTO_CREATE);

        // フリックイベント等のジェスチャーを設定する
        mGestureDetector = new GestureDetector(getApplicationContext(), new OnGestureListener() {
            @Override
            public boolean onDown(MotionEvent e) {
                return false;
            }

            @Override
            public boolean onFling(MotionEvent e1, MotionEvent e2,
                    float velocityX, float velocityY) {
                if (velocityY > 0) {
                    // previous
                    mFlickDirection = Const.FLICK_PREVIOUS;
                } else {
                    // next
                    mFlickDirection = Const.FLICK_NEXT;
                }
                return true;
            }

            @Override
            public void onLongPress(MotionEvent e) {
            }

            @Override
            public boolean onScroll(MotionEvent e1, MotionEvent e2,
                    float distanceX, float distanceY) {
                return false;
            }

            @Override
            public void onShowPress(MotionEvent e) {
            }

            @Override
            public boolean onSingleTapUp(MotionEvent e) {
                return false;
            }
        });

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onCreate() end.");
        }
    }

    /**
     * Unicode絵文字のカテゴリ・文字コードをリソースから取得する
     */
    private void loadUnicodeEmojiResources() {
        mUnicodeEmojiCategoryInfoList = new ArrayList<UnicodeEmojiCategoryInfo>();

        // カテゴリ名取得
        String[] categories = getResources().getStringArray(R.array.category);

        // カテゴリ情報設定
        for (int i = 0; i < categories.length; i++) {
            UnicodeEmojiCategoryInfo categoryInfo = new UnicodeEmojiCategoryInfo();
            categoryInfo.setCategoryName(categories[i]);
            categoryInfo.setmEmojiCodeList(Arrays
                    .asList(UNICODE_EMOJI_TABLES[i]));
            mUnicodeEmojiCategoryInfoList.add(categoryInfo);
        }

        // 入力履歴の存在チェック
        mIsExistUnicodeInputHistory = mInputHistoryDao
                .isExistHistoriesUnicode();

        // 入力履歴が存在する場合
        if (mIsExistUnicodeInputHistory) {
            // カテゴリの1件目を入力履歴から設定する
            UnicodeEmojiCategoryInfo historyInfo = new UnicodeEmojiCategoryInfo();
            historyInfo.setCategoryName(getResources().getString(
                    R.string.input_history_name));
            mUnicodeEmojiCategoryInfoList.add(0, historyInfo);
        }
    }

    /** @see android.app.Activity#onDestroy */
    @Override
    protected void onDestroy() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onDestory() start.");
        }

        super.onDestroy();
        if (mSwitchEmojiFlg == Const.SWITCH_VALUE_BOTH
                || mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            // デコ絵文字
            if (mIDecoEmojiManagerIF != null) {
                unbindService(serviceConn);
            }
        }

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        EmojiSampleApplication obj = EmojiSampleApplication.getInstance();
        if (obj == null) {
            return;
        }
        mEmojiAssist.addView(obj.getEditText());
    }

    /** @see android.app.Activity#onStop */
    @Override
    protected void onStop() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onStop() start.");
        }

        super.onStop();
        sendResultData(false);

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onStop() end.");
        }
    }

    /** @see android.app.Activity#onResume */
    @Override
    protected void onResume() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onResume() start.");
        }

        super.onResume();
        Boolean isDeco = false;
        if (mAddViewList != null && mAddViewList.size() > 0) {
            for (int i = 0; mAddViewList.size() > i; i++) {
                Spannable text = new SpannableString(mAddViewList.get(i)
                        .getText());
                int ret = mEmojiAssist.checkTextData(text);
                if (ret == EmojiAssist.TYPE_TEXT_DECOEMOJI) {
                    isDeco = true;
                    break;
                }
            }
            if (isDeco) {
                mEmojiAssist.startAnimation();
            } else {
                mEmojiAssist.stopAnimation();
            }
        } else {
            mEmojiAssist.stopAnimation();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onResume() end.");
        }
    }

    /** @see android.app.Activity#onPause */
    @Override
    protected void onPause() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onPause() start.");
        }

        super.onPause();
        // スリープ時に描画処理の停止
        mEmojiAssist.stopAnimation();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onPause() end.");
        }
    }

    /** @see android.app.Activity#onConfigurationChanged */
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onConfigurationChanged() start.");
        }

        super.onConfigurationChanged(newConfig);

        // 回転後の情報をコンテントに登録する
        WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
        int width = wm.getDefaultDisplay().getWidth();
        int height = wm.getDefaultDisplay().getHeight();
        Resources res = getResources();
        LayoutInflater inflater = getLayoutInflater();
        ViewGroup viewGroup = mListViewManager.initView(res, inflater, width,
                height, mCurrentImeHeight, mCurrentInputType);
        setContentView(viewGroup);

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        // 画面に配置するコンポーネントの設定を行う
        setComponentEvents();

        // デコ／Unicode絵文字タブの設定を行う
        setSwitchEmojiTabEvent();

        // カテゴリ名称を設定する
        setCategoryName();

        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            // デコ絵文字

            // 回転前のカテゴリ情報のインデックスを記憶する
            mCurrentCategoryDataIndexTemp = mCurrentCategoryDataIndex;

            // スクロールバーを初期化する
            mScrollFrom = 0;
            mScrollTo = 0;

            // カテゴリ情報一覧を更新する
            initCategoryDataList();
        } else {
            // Unicode絵文字

            // 回転前のカテゴリ情報のインデックスを記憶する
            mCurrentUnicodeCategoryDataIndexTemp = mCurrentUnicodeCategoryDataIndex;

            // スクロールバーを初期化する
            mScrollFrom = 0;
            mScrollTo = 0;

            // カテゴリ情報一覧を更新する
            initUnicodeCategoryDataList();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onConfigurationChanged() end.");
        }
    };

    /**
     * サービス接続の確立／切断用メソッドをオーバーライドする.
     */
    ServiceConnection serviceConn = new ServiceConnection() {

        // サービスとの接続が確立された時に実行される
        public void onServiceConnected(ComponentName componentName,
                IBinder iBinder) {
            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "onServiceConnected() start.");
            }

            mIDecoEmojiManagerIF = IDecoEmojiManager.Stub.asInterface(iBinder);

            if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                // デコ絵文字を表示する
                initCategoryDataList();
                // カテゴリ情報がない場合はリトライ
                if (mCategoryInfoArray == null || mCategoryInfoCheck == true) {
                    mCategoryInfoCheck = false;
                    if (mNoPictDData != NO_PICT_D_DATA_FLAG) {
                        retryCheck();
                    }
                }
            } else {
                // Unicode絵文字を表示する
                initUnicodeCategoryDataList();
            }

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "onServiceConnected() end.");
            }
        }

        // サービスとの接続が切断された時に実行される
        public void onServiceDisconnected(ComponentName componentName) {
            mIDecoEmojiManagerIF = null;
        }
    };

    /**
     * カテゴリ情報一覧を更新する処理を行う.
     */
    private void initCategoryDataList() {
        List<DecoEmojiCategoryInfo> categoryInfoList;

        // カテゴリ情報一覧の要素数計算用
        int categoryIdNameListSize = 0;

        mPageInfo = new LinkedHashMap<Integer, PageInfo>();

        categoryInfoList = mInputHistoryDao.getCategoryInfoList(
                mListViewManager, mEmojiType);

        categoryIdNameListSize += categoryInfoList.size();

        // 入力履歴の初期処理を行う
        if (!isFinishing()) {
            mInputHistoryDao
                    .init(mListViewManager.getDecoEmojiMaxDrawableCount(),
                            mEmojiType);
            mIsExistInputHistory = mInputHistoryDao.isExistHistories();
            if (mIsExistInputHistory) {
                categoryIdNameListSize++;
            }
        }

        // カテゴリ情報を取得する
        if (categoryIdNameListSize != 0) {

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME
                        + "initCategoryDataList() categoryIdNameListSize["
                        + categoryIdNameListSize + "]");
            }

            mCategoryInfoArray = new DecoEmojiCategoryInfo[categoryIdNameListSize];
            for (int i = 0; i < categoryIdNameListSize; i++) {
                if (mIsExistInputHistory) {
                    if (i == 0) {
                        // 入力履歴が存在する時、1件目をダミー情報として保存する
                        mCategoryInfoArray[0] = null;
                        mCategoryInfoCheck = true;
                    } else {
                        mCategoryInfoCheck = false;
                        // カテゴリ情報を保存する
                        if (categoryInfoList.get(i - 1) != null) {
                            mCategoryInfoArray[i] = categoryInfoList.get(i - 1);
                        }
                    }
                } else {
                    // カテゴリ情報を保存する
                    if (categoryInfoList.get(i) != null) {
                        mCategoryInfoArray[i] = categoryInfoList.get(i);
                    }
                }
            }
            // 回転前のカテゴリ情報のインデックスを設定する(初回は先頭ページを設定)
            mCurrentCategoryDataIndex = mCurrentCategoryDataIndexTemp;
        } else {
            mCategoryInfoArray = null;
        }

        // カレントページに初期ページを設定する
        mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;

        // カテゴリ名称を設定する
        setCategoryName();

        // カテゴリ情報一覧を設定する
        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            toCurrentPage();
        }
    }

    /**
     * リトライ回数チェック処理
     */
    private void retryCheck() {
        // リトライ回数＝6回
        if (mRetryCount != 0) {
            if (!isFinishing()) {
                retryThread = new Thread(this);
                retryThread.start();
            }
        } else {
            mNoPictDData = NO_PICT_D_DATA_FLAG;
            mRetryCount = MAX_RETRY_COUNT;
            Toast.makeText(getApplicationContext(),
                    R.string.pict_d_result_message, Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * リトライ待ちウェイト処理
     */
    public void run() {
        try {
            Thread.sleep(RETRY_TIME);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        handler.sendEmptyMessage(0);
    }

    /**
     * ウェイト完了後処理
     */
    private Handler handler = new Handler() {
        public void handleMessage(Message msg) {
            mRetryCount--;
            initCategoryDataList();
            if (mCategoryInfoArray == null || mCategoryInfoCheck == true) {
                mCategoryInfoCheck = false;
                if (mNoPictDData != NO_PICT_D_DATA_FLAG) {
                    retryCheck();
                }
            } else {
                mRetryCount = MAX_RETRY_COUNT;
            }
        }
    };

    public String uri;

    /**
     * Unicode絵文字カテゴリ情報一覧を更新する処理を行う.
     */
    private void initUnicodeCategoryDataList() {

        // 回転前のカテゴリ情報のインデックスを設定する(初回は先頭ページを設定)
        mCurrentUnicodeCategoryDataIndex = mCurrentUnicodeCategoryDataIndexTemp;

        // カテゴリ名称を設定する
        setCategoryName();

        // カテゴリ情報一覧を設定する
        if (mUnicodeEmojiCategoryInfoList != null) {
            showUnicodeEmoji(mCurrentUnicodeCategoryDataIndex);
        }
    }

    /**
     * DecoEmojiManagerからデコ絵文字一覧を取得し、画面に表示を行う.
     * 
     * @param targetPage
     *            表示するページ数
     */
    private void showCategoryList(int targetPage) {

        PageInfo pageInfo;

        // カテゴリ情報一覧が無い時、処理をしない
        if (mCategoryInfoArray == null) {
            return;
        }

        if (mPageInfo.containsKey(targetPage)) {
            pageInfo = mPageInfo.get(targetPage);
        } else {
            pageInfo = new PageInfo(0, 0, 0, 0);
        }

        // デコ絵文字一覧を表示するレイアウトを設定する
        LinearLayout targetLinear = new LinearLayout(getApplicationContext());
        targetLinear.setVisibility(View.GONE);
        targetLinear.setOrientation(LinearLayout.VERTICAL);
        targetLinear.setMinimumHeight(mListViewManager
                .getDecoEmojiLinearHeight());
        targetLinear.removeAllViews();

        // 表示するデコ絵文字一覧を取得する
        if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
            mCurrentDecoEmojiList = null;
            // 保持しているページ数を超えていない時に取得を行う
            if (Const.CATEGORY_DATA_FIRST_PAGE <= targetPage
                    && targetPage <= mMaxPage) {
                mCurrentDecoEmojiList = mInputHistoryDao.loadHistories(
                        targetPage, pageInfo.getEmojiStart(), mEmojiType);
            }
            if (mCurrentDecoEmojiList != null) {
                showDecoEmojiList(targetLinear, pageInfo);
                PageInfo nextPageInfo = new PageInfo(0,
                        pageInfo.getEmojiCount() + pageInfo.getPictureCount(),
                        0, 0);
                mPageInfo.put(targetPage + 1, nextPageInfo);
                if (mMaxPage < (targetPage + 1)) {
                    mMaxPage = targetPage + 1;
                }
            }
        } else {
            // DecoEmojiManagerから取得する
            DecoEmojiCategoryInfo categoryInfo = mCategoryInfoArray[mCurrentCategoryDataIndex];
            mCurrentDecoEmojiList = new ArrayList<EmojiAssist.DecoEmojiTextInfo>();

            // Pictureのスタート0の場合、デコ絵文字を取得
            if (pageInfo.getPictureStart() == 0) {
                mCurrentDecoEmojiList = mInputHistoryDao.getDecoEmojiList(
                        mListViewManager, mEmojiType, targetPage, categoryInfo,
                        pageInfo.getEmojiStart(), false);
            }

            // Pictureを取得する。
            if (mListViewManager.getDecoEmojiMaxDrawableCount() > mCurrentDecoEmojiList
                    .size() || pageInfo.getEmojiStart() == 0) {
                List<EmojiAssist.DecoEmojiTextInfo> currentPictureList = mInputHistoryDao
                        .getDecoEmojiList(mListViewManager, mEmojiType,
                                targetPage, categoryInfo,
                                pageInfo.getPictureStart(), true);
                mCurrentDecoEmojiList.addAll(currentPictureList);
            }

            if (mCurrentDecoEmojiList.size() != 0) {
                showDecoEmojiList(targetLinear, pageInfo);
                PageInfo nextPageInfo = new PageInfo(0,
                        pageInfo.getEmojiCount(), 0, pageInfo.getPictureCount());
                mPageInfo.put(targetPage + 1, nextPageInfo);
                if (mMaxPage < (targetPage + 1)) {
                    mMaxPage = targetPage + 1;
                }
            }
        }
        mDecoEmojiListBaseLinear.addView(targetLinear);
    }

    /**
     * デコ絵文字一覧を表示する。
     * 
     * @param targetLinear
     *            デコ絵文字を表示するレイアウト
     * @param pageInfo
     *            レイアウトのページ情報
     */
    private void showDecoEmojiList(LinearLayout targetLinear, PageInfo pageInfo) {

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "showCategoryList() mCurrentDecoEmojiList.size["
                    + mCurrentDecoEmojiList.size() + "]");
        }

        if (mCurrentDecoEmojiList == null || mCurrentDecoEmojiList.size() == 0) {
            return;
        }

        int decoEmojiCounter = 0;
        int decoEmojiCount = 0;
        int decoPictureCount = 0;
        int decoEmojiWidth = 0;
        int decoEmojiHeight = 0;
        TextView decoEmojiView;
        int viewCnt;
        int preEmojiPosition;

        // 画像の縦横を取得
        int imageHeight = Const.DEFAULT_HEIGHT;
        int imageWidth = Const.DEFAULT_WIDTH;
        int kind = Const.DEFAULT_KIND;
        boolean changeRowFlag = false;

        // 全体のデコ絵文字を表示する(行)
        for (int i = 0; i < mListViewManager.getDecoEmojiMaxRows(); i++) {
            mDecoEmojiListInner = new LinearLayout(getApplicationContext());
            mDecoEmojiListInner.setOrientation(LinearLayout.HORIZONTAL);
            viewCnt = 0;
            preEmojiPosition = 0;
            // 1行分のデコ絵文字を表示する(列)
            for (int j = 0; j < mListViewManager.getDecoEmojiMaxCols()
                    && decoEmojiCounter < mCurrentDecoEmojiList.size(); j++) {

                // 設定クラスを生成
                EmojiAssist.DecoEmojiTextInfo decoEmojiTextInfo = new EmojiAssist.DecoEmojiTextInfo();
                EmojiAssist.DecoEmojiTextInfo currentDecoEmojiInfo = mCurrentDecoEmojiList
                        .get(decoEmojiCounter);
                // コンテンツURI
                decoEmojiTextInfo.setUri(Const.PREFIX_URI
                        + currentDecoEmojiInfo.getUri());
                // 対象絵文字タイプ(値が0以下の場合は下位互換のためAnnotationがURIのみとなる)
                decoEmojiTextInfo.setContext(getApplicationContext());
                if (decoEmojiWidth > 0 && decoEmojiHeight > 0) {
                    // 絵文字情報を利用者側で取得済みの場合は設定クラスに情報をセット
                    decoEmojiTextInfo.setWidth(currentDecoEmojiInfo.getWidth());
                    decoEmojiTextInfo.setHeight(currentDecoEmojiInfo.getHeight());
                    decoEmojiTextInfo.setKind(currentDecoEmojiInfo.getKind());
                    decoEmojiTextInfo.setPop(currentDecoEmojiInfo.getPop());
                }
                decoEmojiTextInfo.setEmojiType(mEmojiType);

                CharSequence charSequence = mEmojiAssist
                        .getDecoEmojiText(decoEmojiTextInfo);

                decoEmojiView = new TextView(getApplicationContext());
                decoEmojiView.setTextSize(DECOEMOJI_SIZE);
                float textSize = decoEmojiView.getTextSize();

                imageWidth = currentDecoEmojiInfo.getWidth();
                imageHeight = currentDecoEmojiInfo.getHeight();
                kind = currentDecoEmojiInfo.getKind();
                if (imageHeight > Const.DEFAULT_HEIGHT) {
                    int resizeHeight = Const.DEFAULT_HEIGHT;
                    int resizeWidth = imageWidth / imageHeight
                            * Const.DEFAULT_HEIGHT;
                    if (resizeWidth == 0) {
                        resizeWidth = Const.DEFAULT_WIDTH;
                    }
                    imageHeight = resizeHeight;
                    imageWidth = resizeWidth;
                }

                imageHeight = (int) ((float) imageHeight * textSize / Const.DEFAULT_WIDTH);
                imageWidth = (int) ((float) imageWidth * textSize / Const.DEFAULT_WIDTH)
                        + Const.TEXT_PADDING_LEFT + Const.TEXT_PADDING_RIGHT;

                // 1画面の最終デコ絵文字は縦幅を変更する
                if ((i + 1) == mListViewManager.getDecoEmojiMaxRows()) {
                    decoEmojiHeight = mListViewManager
                            .getDecoEmojiHeightEndOfLine();
                } else {
                    decoEmojiHeight = mListViewManager.getDecoEmojiHeight();
                }

                int length = (int) Math.floor((double) imageWidth
                        / mListViewManager.getDecoEmojiWidth());
                if (imageWidth % mListViewManager.getDecoEmojiWidth() > 0) {
                    length++;
                }

                if (length >= mListViewManager.getDecoEmojiMaxCols()) {
                    length = mListViewManager.getDecoEmojiMaxCols();
                }

                // デコメピクチャが来た場合は改行する
                if (mCategoryInfoArray[mCurrentCategoryDataIndex] != null
                        && kind == DecoEmojiContract.KIND_PICTURE
                        && !changeRowFlag) {
                    changeRowFlag = true;
                    if (viewCnt > 0) {
                        // 改行処理
                        updatePreViewLayOut(viewCnt, decoEmojiCounter,
                                preEmojiPosition, decoEmojiHeight);
                        break;
                    }
                }

                // 絵文字が全体の行の長さを超える場合は改行する
                if (j + length > mListViewManager.getDecoEmojiMaxCols()) {
                    if (viewCnt > 0) {
                        // 改行処理
                        updatePreViewLayOut(viewCnt, decoEmojiCounter,
                                preEmojiPosition, decoEmojiHeight);
                        break;
                    }
                }

                // 開始位置移動
                preEmojiPosition = j;
                j = j + length - 1;
                decoEmojiWidth = length * mListViewManager.getDecoEmojiWidth();

                if (j == mListViewManager.getDecoEmojiMaxCols()) {
                    decoEmojiWidth = decoEmojiWidth
                            + (mListViewManager.getDisplayWidth() % mListViewManager
                                    .getDecoEmojiMaxCols());
                }
                decoEmojiView = getCategoryTextView(decoEmojiView,
                        decoEmojiCounter, decoEmojiWidth, decoEmojiHeight,
                        charSequence);
                mDecoEmojiListInner.addView(decoEmojiView);
                decoEmojiCounter++;
                viewCnt++;

                if (kind != DecoEmojiContract.KIND_PICTURE) {
                    decoEmojiCount++;
                } else {
                    decoPictureCount++;
                }

            }
            targetLinear.addView(mDecoEmojiListInner);
        }
        targetLinear.setVisibility(View.VISIBLE);

        pageInfo.setEmojiCount(decoEmojiCount + pageInfo.getEmojiStart());
        pageInfo.setPictureCount(decoPictureCount + pageInfo.getPictureStart());
    }

    /**
     * 画面のボタンのイベント設定、カテゴリ名称設定、プレビューテキスト設定を行う.
     */
    private void setComponentEvents() {
        // コンポーネントを定義する
        setComponentFindViewById();

        // プレビューテキスト
        setPreviewText();

        // 確定ボタン
        setEnterButtonEvent();

        // スクロールバー
        setBaseScrollEvent();

        // カテゴリ内ページ切替(上)ボタン
        setReadMoreButtonBackEvent();

        // カテゴリ内ページ切替(下)ボタン
        setReadMoreButtonNextEvent();

        // バックスペースボタン
        setBackSpaceButtonEvent();
    }

    /**
     * コンポーネントを定義する.
     */
    private void setComponentFindViewById() {
        mSwitchEmojiTabDeco = (TextView) findViewById(R.id.switch_emoji_tab_deco_text);
        mSwitchEmojiTabUnicode = (TextView) findViewById(R.id.switch_emoji_tab_unicode_text);
        mPreviewText = (TextView) findViewById(R.id.preview_text);
        mPreviewText.setTextSize(DECOEMOJI_SIZE);
        Bundle bundle = mPreviewText.getInputExtras(true);
        bundle.putBoolean("tagsSensible", true);
        // 1個確定モードの場合はプレビューテキストエリアを表示しない
        if (mCurrentInputType == INPUT_TYPE_SINGLE) {
            mPreviewText.setVisibility(View.GONE);
        }
        mBaseScroll = (ScrollView) findViewById(R.id.base_scroll);
        mDecoEmojiListBaseLinear = (LinearLayout) findViewById(R.id.base_list_linear);
        mEnterButton = (ImageView) findViewById(R.id.enter_button);
        mReadMoreButtonBack = (ImageView) findViewById(R.id.read_more_back_button);
        mReadMoreButtonNext = (ImageView) findViewById(R.id.read_more_next_button);
        mBackSpaceButton = (ImageView) findViewById(R.id.back_space_button);

        // 1個確定モードの場合はボタンエリア、及びボタンを表示しない
        if (mCurrentInputType == INPUT_TYPE_SINGLE) {
            mBaseButtonsLinear = (LinearLayout) findViewById(R.id.base_buttons_linear);
            mBaseButtonsLinear.setVisibility(View.GONE);
        }

        mCategoryName = (Spinner) findViewById(R.id.category_name_spinner);
    }

    /**
     * IMEに返却する文字列を画面上部のプレビューテキストに設定する.
     */
    private void setPreviewText() {
        if (mReturnValue.size() != 0) {
            mPreviewText.setText("");
            DecoEmojiListResultInfo decoEmojiListResultInfo;
            boolean decoEmojiFlg = false;
            boolean unicodeEmojiFlg = false;
            for (int i = 0; i < mReturnValue.size(); i++) {
                decoEmojiListResultInfo = mReturnValue.get(i);

                CharSequence charSequence;
                switch (decoEmojiListResultInfo.getTag()) {
                // デコ絵文字
                case Const.SWITCH_VALUE_DECO:
                    EmojiAssist.DecoEmojiTextInfo decoEmojiTextInfo = new EmojiAssist.DecoEmojiTextInfo();
                    decoEmojiTextInfo.setUri(Const.PREFIX_URI
                            + decoEmojiListResultInfo.getUri());
                    decoEmojiTextInfo.setContext(getApplicationContext());
                    decoEmojiTextInfo.setEmojiType(mEmojiType);
                    charSequence = mEmojiAssist
                            .getDecoEmojiText(decoEmojiTextInfo);
                    decoEmojiFlg = true;
                    break;

                // Unicode絵文字
                case Const.SWITCH_VALUE_UNICODE:
                    charSequence = decoEmojiListResultInfo.getCode();
                    unicodeEmojiFlg = true;
                    break;

                default:
                    throw new IllegalArgumentException("Unsupported tag "
                            + decoEmojiListResultInfo.getTag());

                }

                mPreviewText.append(charSequence);
            }
            int addViewKind = 0;
            if (decoEmojiFlg && unicodeEmojiFlg) {
                addViewKind = Const.SWITCH_VALUE_BOTH;
            } else if (decoEmojiFlg) {
                addViewKind = Const.SWITCH_VALUE_DECO;
            } else if (unicodeEmojiFlg) {
                addViewKind = Const.SWITCH_VALUE_UNICODE;
            } else {
                return;
            }
            addView(mPreviewText, false, addViewKind);
        }
    }

    /**
     * デコ／Unicode絵文字タブのイベント設定を行う.
     */
    private void setSwitchEmojiTabEvent() {
        if (mSwitchEmojiFlg == Const.SWITCH_VALUE_UNICODE) {
            // デコ絵文字タブ：無効 Unicode絵文字タブ：有効
            mSwitchEmojiTabDeco.setVisibility(View.GONE);
            mSwitchEmojiTabUnicode
                    .setBackgroundResource(R.drawable.switch_emoji_tab);
            mSwitchEmojiTabUnicode.setMinimumWidth(mListViewManager
                    .getDisplayWidth());
            mSwitchEmojiTabUnicode.setTextColor(Color.WHITE);
            mSwitchEmojiTabUnicode.setVisibility(View.VISIBLE);
        } else {
            // デコ絵文字タブ：有効 Unicode絵文字タブ：有効
            mSwitchEmojiTabDeco
                    .setBackgroundResource(R.drawable.switch_emoji_tab);
            mSwitchEmojiTabDeco.setMinimumWidth((mListViewManager
                    .getDisplayWidth() / 2));
            mSwitchEmojiTabDeco.setOnClickListener(mSwitchEmojiTabOnClick);
            mSwitchEmojiTabDeco.setTextColor(Color.WHITE);
            mSwitchEmojiTabDeco.setVisibility(View.VISIBLE);
            mSwitchEmojiTabUnicode
                    .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
            mSwitchEmojiTabUnicode.setMinimumWidth((mListViewManager
                    .getDisplayWidth() / 2));
            mSwitchEmojiTabUnicode.setOnClickListener(mSwitchEmojiTabOnClick);
            mSwitchEmojiTabUnicode.setTextColor(Color.GRAY);
            mSwitchEmojiTabUnicode.setVisibility(View.VISIBLE);
            if (mCurrentEmojiValue == Const.SWITCH_VALUE_UNICODE) {
                // デコ絵文字タブを非活性にする
                mSwitchEmojiTabDeco
                        .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
                mSwitchEmojiTabDeco.setTextColor(Color.GRAY);
                // Unicode絵文字タブを活性にする
                mSwitchEmojiTabUnicode
                        .setBackgroundResource(R.drawable.switch_emoji_tab);
                mSwitchEmojiTabUnicode.setTextColor(Color.WHITE);
            }
        }
    }

    /**
     * 確定ボタンのイベント設定を行う.
     */
    private void setEnterButtonEvent() {
        mEnterButton.setOnClickListener(new View.OnClickListener() {
            /**
             * 確定ボタン押下時のイベント
             */
            public void onClick(View view) {

                // 入力履歴をデータベースに保存する
                if (mReturnValue.size() != 0) {
                    DecoEmojiListResultInfo decoEmojiListResultInfo;

                    for (int i = 0; i < mReturnValue.size(); i++) {
                        decoEmojiListResultInfo = mReturnValue.get(i);

                        switch (decoEmojiListResultInfo.getTag()) {
                        // デコ絵文字
                        case Const.SWITCH_VALUE_DECO:
                            try {
                                // 利用履歴情報の同期(履歴情報を更新)
                                mIDecoEmojiManagerIF
                                        .aidl_changeHistory(decoEmojiListResultInfo
                                                .getUri());
                            } catch (RemoteException e) {
                                e.printStackTrace();
                            }
                            break;

                        // Unicode絵文字
                        case Const.SWITCH_VALUE_UNICODE:
                            mInputHistoryDao
                                    .updateHistoryUnicode(decoEmojiListResultInfo
                                            .getCode());
                            break;

                        default:
                            throw new IllegalArgumentException(
                                    "Unsupported tag "
                                            + decoEmojiListResultInfo.getTag());
                        }

                    }
                }

                if (Const.DEBUG) {
                    Log.d(Const.TAG,
                            CLASS_NAME
                                    + "setEnterButtonEvent.onClick() mReturnValue.size["
                                    + mReturnValue.size() + "]");
                }

                // 結果を返却する
                sendResultData(true);
            }
        });
    }

    /**
     * スクロールバーのイベント設定を行う.
     */
    private void setBaseScrollEvent() {
        // スクロールバー(Y方向)を不可視にする
        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            mBaseScroll.setVerticalScrollBarEnabled(false);
        }

        mBaseScroll.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {

                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // デコ絵文字

                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return false;
                    }
                    // フリック(orフリング)中は処理しない
                    if (mIsFlick == false) {
                        mIsFlick = mGestureDetector.onTouchEvent(event);
                    }

                    switch (event.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        mIsFlick = false;
                        break;
                    case MotionEvent.ACTION_MOVE:
                        mIsFlick = false;
                        break;
                    case MotionEvent.ACTION_UP:
                        if (mIsFlick) {
                            switch (mFlickDirection) {
                            // 下方向にフリックする
                            case Const.FLICK_PREVIOUS:
                                shiftPreviousPage();
                                break;
                            // 上方向にフリックする
                            case Const.FLICK_NEXT:
                                shiftNextPage();
                                break;
                            default:
                                break;
                            }
                        } else if (mScrollTo > mBaseScroll.getScrollY()) {
                            // スクロールバーが下に移動している
                            shiftPreviousPage();
                        } else if (mScrollTo < mBaseScroll.getScrollY()) {
                            // スクロールバーが上に移動している
                            shiftNextPage();
                        }
                        break;
                    }
                    return false;

                } else {
                    // Unicode絵文字では、スクロールバーの制御はしない
                    return false;
                }
            }
        });
    }

    /** 上ページに移動する為、メンバ変数を更新する. */
    private void shiftPreviousPage() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftPreviousPage() start.");
        }

        // 先頭ページは前ページに移動しない
        if (mCurrentCategoryDataPage != Const.CATEGORY_DATA_FIRST_PAGE) {
            mCurrentCategoryDataPage--;

            // 移動開始位置を調節する
            if (mCurrentCategoryDataPage == Const.CATEGORY_DATA_FIRST_PAGE) {
                mScrollFrom = mBaseScroll.getScrollY();
                mScrollTo = 0;
            } else {
                mScrollFrom = mBaseScroll.getScrollY()
                        + mListViewManager.getDecoEmojiLinearHeight();
                mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
            }

            toPreviousScroll();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftPreviousPage() end.");
        }
    }

    /** 下ページに移動する為、メンバ変数を更新する. */
    private void shiftNextPage() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftNextPage() start.");
        }

        int currentMaxPageSize = mMaxPage;

        // 最終ページは次ページに移動しない
        if ((mCurrentCategoryDataPage + 1) != currentMaxPageSize) {
            mCurrentCategoryDataPage++;

            // 移動開始位置を調節する
            if ((mCurrentCategoryDataPage + 1) != currentMaxPageSize) {
                mScrollFrom = mBaseScroll.getScrollY()
                        - mListViewManager.getDecoEmojiLinearHeight();
            } else {
                mScrollFrom = mBaseScroll.getScrollY();
            }

            mScrollTo = mListViewManager.getDecoEmojiLinearHeight();

            toNextScroll();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftNextPage() end.");
        }
    }

    /**
     * デコ／Unicode絵文字切替テキストのonClickイベントの設定を行う.
     */
    private final OnClickListener mSwitchEmojiTabOnClick = new View.OnClickListener() {
        /**
         * デコ／Unicode絵文字切替テキスト押下時のイベント
         */
        public void onClick(View v) {
            if (v instanceof TextView) {
                TextView textView = (TextView) v;
                switch (textView.getId()) {
                // デコ絵文字タブ押下
                case (R.id.switch_emoji_tab_deco_text):
                    if (mCurrentEmojiValue == Const.SWITCH_VALUE_UNICODE) {
                        mSwitchEmojiTabDeco
                                .setBackgroundResource(R.drawable.switch_emoji_tab);
                        mSwitchEmojiTabDeco.setTextColor(Color.WHITE);
                        mSwitchEmojiTabUnicode
                                .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
                        mSwitchEmojiTabUnicode.setTextColor(Color.GRAY);
                        mCurrentEmojiValue = Const.SWITCH_VALUE_DECO;

                        // スクロールバー(Y方向)を不可視にする
                        mBaseScroll.setVerticalScrollBarEnabled(false);
                        // デコ絵文字を表示する
                        initCategoryDataList();
                    }
                    break;

                // Unicode絵文字タブ押下
                case (R.id.switch_emoji_tab_unicode_text):
                    if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                        mSwitchEmojiTabDeco
                                .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
                        mSwitchEmojiTabDeco.setTextColor(Color.GRAY);
                        mSwitchEmojiTabUnicode
                                .setBackgroundResource(R.drawable.switch_emoji_tab);
                        mSwitchEmojiTabUnicode.setTextColor(Color.WHITE);
                        mCurrentCategoryDataIndexTemp = 0;
                        mCurrentUnicodeCategoryDataIndexTemp = 0;
                        mCurrentEmojiValue = Const.SWITCH_VALUE_UNICODE;

                        // スクロールバー(Y方向)を可視にする
                        mBaseScroll.setVerticalScrollBarEnabled(true);

                        clearDecoComponents();
                        initUnicodeCategoryDataList();
                    }
                    break;

                default:
                    break;
                }
            }
        }
    };

    /**
     * カテゴリ内ページ切替(上)ボタンのイベント設定を行う.
     */
    private void setReadMoreButtonBackEvent() {

        mReadMoreButtonBack.setOnClickListener(new View.OnClickListener() {
            /**
             * カテゴリ内ページ切替(上)ボタン押下時のイベント.
             */
            public void onClick(View v) {

                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // デコ絵文字

                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return;
                    }

                    // 先頭ページである時は最終ページを表示する
                    int currentMaxPageSize = mMaxPage;

                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return;
                    }

                    if (mCurrentCategoryDataPage == Const.CATEGORY_DATA_FIRST_PAGE) {
                        // 1ページの場合はなにもなしない。
                        return;
                    } else {
                        // 前ページを設定する
                        mCurrentCategoryDataPage--;
                        mScrollFrom = mBaseScroll.getScrollY()
                                + mListViewManager.getDecoEmojiLinearHeight();
                        mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
                        if (mCurrentCategoryDataPage == Const.CATEGORY_DATA_FIRST_PAGE) {
                            mScrollFrom = mListViewManager
                                    .getDecoEmojiLinearHeight();
                            mScrollTo = 0;
                        }
                        toPreviousScroll();
                    }

                } else {
                    // Unicode絵文字

                    if (mBaseScroll.getScrollY() == 0) {
                        // 先頭の場合には何もしない。
                        return;
                    }
                    if (!mBaseScroll.pageScroll(ScrollView.FOCUS_UP)) {
                        mBaseScroll.scrollTo(0, mBaseScroll.getChildAt(0)
                                .getHeight());
                    }
                }
            }
        });

        mReadMoreButtonBack.setOnLongClickListener(new View.OnLongClickListener() {
            /**
             * カテゴリ内ページ切替(上)ボタン長押し時のイベント.
             */
            public boolean onLongClick(View v) {
                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // デコ絵文字

                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return true;
                    }

                    int currentMaxPageSize = mMaxPage;

                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return true;
                    }

                    // 先頭ページでなければ
                    if (mCurrentCategoryDataPage != Const.CATEGORY_DATA_FIRST_PAGE) {
                        // 先頭ページを設定する
                        mScrollFrom = 0;
                        mScrollTo = 0;
                        mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;
                        // mCurrentCategoryDataPageを元にデコ絵文字一覧を表示する
                        toCurrentPage();
                    }
                } else {
                    // Unicode絵文字

                    if (mBaseScroll.getScrollY() == 0) {
                        // 先頭の場合には何もしない。
                        return true;
                    }
                    mBaseScroll.fullScroll(ScrollView.FOCUS_UP);
                }

                return true;
            }
        });
    }

    /**
     * カテゴリ内ページ切替(下)ボタンのイベント設定を行う.
     */
    private void setReadMoreButtonNextEvent() {

        mReadMoreButtonNext.setOnClickListener(new View.OnClickListener() {
            /**
             * カテゴリ内ページ切替(下)ボタン押下時のイベント.
             */
            public void onClick(View v) {

                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // デコ絵文字

                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return;
                    }

                    // 最終ページである時は先頭ページを表示する
                    int currentMaxPageSize = mMaxPage;

                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return;
                    }

                    if ((mCurrentCategoryDataPage + 1) == currentMaxPageSize) {
                        // 最終ページの場合はなにもしない。
                    } else {
                        // 次ページを設定する
                        mCurrentCategoryDataPage++;
                        mScrollFrom = mBaseScroll.getScrollY()
                                - mListViewManager.getDecoEmojiLinearHeight();
                        mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
                        toNextScroll();
                    }
                } else {
                    // Unicode絵文字

                    int scrollY = mBaseScroll.getScrollY();
                    int height = mBaseScroll.getHeight();
                    int count = mBaseScroll.getChildCount();
                    if (count > 0) {
                        View view = mBaseScroll.getChildAt(count - 1);
                        int top = scrollY + height;
                        int bottom = view.getBottom();
                        if (top >= bottom) {
                            // 末尾の場合には何もしない。
                            return;
                        }
                    }
                    if (!mBaseScroll.pageScroll(ScrollView.FOCUS_DOWN)) {
                        mBaseScroll.scrollTo(0, 0);
                    }
                }
            }
        });

        mReadMoreButtonNext.setOnLongClickListener(new View.OnLongClickListener() {
            /**
             * カテゴリ内ページ切替(下)ボタン長押し時のイベント.
             */
            public boolean onLongClick(View v) {

                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // デコ絵文字

                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return true;
                    }

                    int currentMaxPageSize = mMaxPage;

                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return true;
                    }

                    // 最終ページでなければ
                    if ((mCurrentCategoryDataPage + 1) != currentMaxPageSize) {
                        // 最終ページを設定する
                        mScrollFrom = mListViewManager
                                .getDecoEmojiLinearHeight();
                        mScrollTo = mListViewManager
                                .getDecoEmojiLinearHeight();
                        mCurrentCategoryDataPage = currentMaxPageSize - 1;
                        // mCurrentCategoryDataPageを元にデコ絵文字一覧を表示する
                        toCurrentPage();
                    }
                } else {
                    // Unicode絵文字

                    int scrollY = mBaseScroll.getScrollY();
                    int height = mBaseScroll.getHeight();
                    int count = mBaseScroll.getChildCount();
                    if (count > 0) {
                        View view = mBaseScroll.getChildAt(count - 1);
                        int top = scrollY + height;
                        int bottom = view.getBottom();
                        if (top >= bottom) {
                            // 末尾の場合には何もしない。
                            return true;
                        }
                    }
                    mBaseScroll.fullScroll(ScrollView.FOCUS_DOWN);
                }

                return true;
            }
        });
    }

    /**
     * バックスペースボタンの設定を行う.
     */
    private void setBackSpaceButtonEvent() {
        mBackSpaceButton.setOnClickListener(new View.OnClickListener() {
            /**
             * バックスペースボタン押下時のイベント.
             */
            public void onClick(View v) {

                ArrayList<DecoEmojiListResultInfo> returnValueDummy = new ArrayList<DecoEmojiListResultInfo>();
                mPreviewText.setText("");
                DecoEmojiListResultInfo decoEmojiListResultInfo;

                // 最後の文字は使用しない
                for (int i = 0; i < (mReturnValue.size() - 1); i++) {
                    decoEmojiListResultInfo = mReturnValue.get(i);

                    CharSequence charSequence;
                    switch (decoEmojiListResultInfo.getTag()) {
                    // デコ絵文字
                    case Const.SWITCH_VALUE_DECO:
                        EmojiAssist.DecoEmojiTextInfo decoEmojiTextInfo = new EmojiAssist.DecoEmojiTextInfo();
                        decoEmojiTextInfo.setUri(Const.PREFIX_URI + decoEmojiListResultInfo.getUri());
                        decoEmojiTextInfo.setContext(getApplicationContext());
                        decoEmojiTextInfo.setEmojiType(mEmojiType);

                        charSequence = mEmojiAssist
                                .getDecoEmojiText(decoEmojiTextInfo);
                        break;

                    // Unicode絵文字
                    case Const.SWITCH_VALUE_UNICODE:
                        charSequence = decoEmojiListResultInfo.getCode();
                        break;

                    default:
                        throw new IllegalArgumentException("Unsupported tag "
                                + decoEmojiListResultInfo.getTag());
                    }

                    mPreviewText.append(charSequence);
                    returnValueDummy.add(decoEmojiListResultInfo);
                }
                mReturnValue = returnValueDummy;
            }
        });
    }

    /**
     * カテゴリ名称の設定を行う.
     */
    private void setCategoryName() {
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(
                getApplicationContext(), R.layout.spinner_item);
        adapter.setDropDownViewResource(R.layout.select_dialog_singlechoice);

        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            // デコ絵文字

            // カテゴリ名称一覧を追加する
            if (mCategoryInfoArray != null) {
                for (int i = 0; i < mCategoryInfoArray.length; i++) {
                    if (mIsExistInputHistory && i == 0) {
                        // 1件目は入力履歴から設定する
                        adapter.add(getResources().getString(
                                R.string.input_history_name));
                    } else {
                        // カテゴリ情報から設定する
                        if (mIsLanguageJapan) {
                            adapter.add(mCategoryInfoArray[i]
                                    .getCategoryName_jpn());
                        } else {
                            adapter.add(mCategoryInfoArray[i]
                                    .getCategoryName_eng());
                        }
                    }
                }
            } else {
                // リトライ処理終了済み
                if (mNoPictDData == NO_PICT_D_DATA_FLAG) {
                    adapter.add(getString(R.string.pict_d_result_message));
                }

                // リトライ処理中
                if (mRetryCount == MAX_RETRY_COUNT
                        && mNoPictDData != NO_PICT_D_DATA_FLAG) {
                    adapter.add(getString(R.string.pict_d_reading_message));
                } else if (mRetryCount == (MAX_RETRY_COUNT - 1)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub));
                } else if (mRetryCount == (MAX_RETRY_COUNT - 2)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                } else if (mRetryCount == (MAX_RETRY_COUNT - 3)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                } else if (mRetryCount == (MAX_RETRY_COUNT - 4)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                } else if (mRetryCount == (MAX_RETRY_COUNT - 5)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                } else if (mRetryCount == 0) {
                    adapter.add(getString(R.string.pict_d_result_message));
                    adapter.remove(getString(R.string.pict_d_reading_message_sub));
                }
            }

            // カテゴリ名称一覧を設定します
            mCategoryName.setAdapter(adapter);
            mCategoryName.setSelection(mCurrentCategoryDataIndex);
            mCategoryName
                    .setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                        /**
                         * カテゴリ名称が選択された時に呼び出されるイベント.
                         */
                        public void onItemSelected(AdapterView<?> parent,
                                View view, int position, long id) {
                            if (mCurrentCategoryDataIndex != position) {
                                mMaxPage = 0;
                                mCurrentCategoryDataIndex = position;
                                mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;
                                mScrollFrom = 0;
                                mScrollTo = 0;

                                toCurrentPage();
                            }
                        }

                        /**
                         * 何も選択されなかった時に呼び出されるイベント.
                         */
                        public void onNothingSelected(AdapterView<?> parent) {
                        }
                    });

        } else {
            // Unicode絵文字

            // カテゴリ名称一覧を追加する
            if (mUnicodeEmojiCategoryInfoList != null) {
               for (UnicodeEmojiCategoryInfo info : mUnicodeEmojiCategoryInfoList) {
                    adapter.add(info.getCategoryName());
                }
            } else {
                adapter.add("");
            }

            // カテゴリ名称一覧を設定します
            mCategoryName.setAdapter(adapter);
            mCategoryName.setSelection(mCurrentUnicodeCategoryDataIndex);
            mCategoryName.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                /**
                 * カテゴリ名称が選択された時に呼び出されるイベント.
                 */
                public void onItemSelected(AdapterView<?> parent,
                        View view, int position, long id) {
                    if (mCurrentUnicodeCategoryDataIndex != position) {
                        mCurrentUnicodeCategoryDataIndex = position;
                        mScrollFrom = 0;
                        mScrollTo = 0;

                        showUnicodeEmoji(mCurrentUnicodeCategoryDataIndex);
                    }
                }

                /**
                 * 何も選択されなかった時に呼び出されるイベント.
                 */
                public void onNothingSelected(AdapterView<?> parent) {
                }
            });
        }
    }

    /**
     * 指定したカテゴリに紐付くUnicode絵文字一覧を表示する
     * 
     * @param categoryIndex
     */
    private void showUnicodeEmoji(int categoryIndex) {

        mDecoEmojiListBaseLinear.removeAllViews();

        mAddViewList.clear();

        // カテゴリ情報一覧が無い時、処理をしない
        if ((mUnicodeEmojiCategoryInfoList == null)
                || (mUnicodeEmojiCategoryInfoList.isEmpty())) {
            return;
        }

        // Unicode絵文字一覧を表示するレイアウトを設定する
        LinearLayout targetLinear = new LinearLayout(getApplicationContext());
        targetLinear.setVisibility(View.GONE);
        targetLinear.setOrientation(LinearLayout.VERTICAL);
        targetLinear.setMinimumHeight(mListViewManager
                .getDecoEmojiLinearHeight());
        targetLinear.removeAllViews();

        // 入力履歴に含まれる絵文字コードを取得する
        if (mIsExistUnicodeInputHistory && categoryIndex == 0) {
            List<EmojiAssist.DecoEmojiTextInfo> unicodeHistories = mInputHistoryDao
                    .loadHistoriesUnicode();
            if (unicodeHistories == null) {
                return;
            }
            List<String> unicodeInputHistories = new ArrayList<String>();
            for (int k = 0; k < unicodeHistories.size(); k++) {
                unicodeInputHistories.add(k, unicodeHistories.get(k).getUri());
            }
            mUnicodeEmojiCategoryInfoList.get(categoryIndex).setmEmojiCodeList(
                    unicodeInputHistories);
        }

        // 選択されたカテゴリの絵文字コードを取得
        UnicodeEmojiCategoryInfo categoryInfo = mUnicodeEmojiCategoryInfoList
                .get(categoryIndex);
        List<String> emojiList = categoryInfo.getmEmojiCodeList();

        if (emojiList != null && emojiList.size() != 0) {

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME
                        + "showCategoryList() mCurrentDecoEmojiList.size["
                        + mCurrentDecoEmojiList.size() + "]");
            }

            int decoEmojiCounter = 0;
            int decoEmojiWidth;
            int decoEmojiHeight;
            TextView decoEmojiView;
            // 全体のUnicode絵文字を表示する(行)
            while (decoEmojiCounter < emojiList.size()) {

                mDecoEmojiListInner = new LinearLayout(getApplicationContext());
                mDecoEmojiListInner.setOrientation(LinearLayout.HORIZONTAL);

                // 1行分のUnicode絵文字を表示する(列)
                for (int j = 0; j < mListViewManager.getDecoEmojiMaxCols()
                        && decoEmojiCounter < emojiList.size(); j++) {
                    decoEmojiView = new TextView(getApplicationContext());
                    // 1行の最終Unicode絵文字は横幅を変更する
                    if ((j + 1) == mListViewManager.getDecoEmojiMaxCols()) {
                        decoEmojiWidth = mListViewManager
                                .getDecoEmojiWidthEndOfLine();
                    } else {
                        decoEmojiWidth = mListViewManager.getDecoEmojiWidth();
                    }

                    decoEmojiHeight = mListViewManager.getDecoEmojiHeight();

                    decoEmojiView
                            .setBackgroundResource(R.drawable.buttons_background_type1);
                    decoEmojiView.setLayoutParams(new GridView.LayoutParams(
                            decoEmojiWidth, decoEmojiHeight));
                    decoEmojiView.setPadding(0, 0, 0, 0);
                    decoEmojiView.setGravity(Gravity.CENTER_HORIZONTAL
                            | Gravity.CENTER_VERTICAL);
                    decoEmojiView.setOnClickListener(mUnicodeEmojiOnClick);
                    decoEmojiView.setTag(emojiList.get(decoEmojiCounter)
                            .toString());

                    decoEmojiView.setText(emojiList.get(decoEmojiCounter)
                            .toString());

                    // 一覧表示される絵文字サイズを大きくする
                    decoEmojiView.setTextSize(DECOEMOJI_SIZE);

                    addView(decoEmojiView, true, Const.SWITCH_VALUE_UNICODE);

                    mDecoEmojiListInner.addView(decoEmojiView);
                    decoEmojiCounter++;
                }
                targetLinear.addView(mDecoEmojiListInner);
            }
            targetLinear.setVisibility(View.VISIBLE);
        }

        mDecoEmojiListBaseLinear.addView(targetLinear);

        setPreviewText();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * mCurrentCategoryDataPageを元にデコ絵文字一覧を表示する.
     */
    private void toCurrentPage() {
        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        showCategoryList(mCurrentCategoryDataPage - 1);
        showCategoryList(mCurrentCategoryDataPage);
        showCategoryList(mCurrentCategoryDataPage + 1);

        setPreviewText();

        startAnimation();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * 上ページに遷移する処理を行う.
     */
    private void toPreviousScroll() {
        LinearLayout linear1st = (LinearLayout) mDecoEmojiListBaseLinear
                .getChildAt(0);
        LinearLayout linear2nd = (LinearLayout) mDecoEmojiListBaseLinear
                .getChildAt(1);
        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        showCategoryList(mCurrentCategoryDataPage - 1);
        mDecoEmojiListBaseLinear.addView(linear1st);
        mDecoEmojiListBaseLinear.addView(linear2nd);

        setDecoEmojiViewToEmojiAssist(linear1st);
        setDecoEmojiViewToEmojiAssist(linear2nd);

        setPreviewText();

        startAnimation();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * 下ページに遷移する処理を行う.
     */
    private void toNextScroll() {
        LinearLayout linear2nd = (LinearLayout) mDecoEmojiListBaseLinear
                .getChildAt(1);
        LinearLayout linear3rd = (LinearLayout) mDecoEmojiListBaseLinear
                .getChildAt(2);
        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        mDecoEmojiListBaseLinear.addView(linear2nd);
        mDecoEmojiListBaseLinear.addView(linear3rd);
        showCategoryList(mCurrentCategoryDataPage + 1);

        setDecoEmojiViewToEmojiAssist(linear2nd);
        setDecoEmojiViewToEmojiAssist(linear3rd);

        setPreviewText();

        startAnimation();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * レイアウトが持つデコ絵文字一覧をEmojiAssistに登録する.
     * 
     * @param target
     *            デコ絵文字一覧を持つレイアウト
     */
    private void setDecoEmojiViewToEmojiAssist(LinearLayout rows) {
        for (int i = 0; i < rows.getChildCount(); i++) {
            LinearLayout cols = (LinearLayout) rows.getChildAt(i);
            for (int j = 0; j < cols.getChildCount(); j++) {
                addView((TextView) cols.getChildAt(j), true,
                        Const.SWITCH_VALUE_DECO);
            }
        }
    }

    /**
     * デコ絵文字一覧に表示させるTextViewの設定を行う.
     * 
     * @param textView
     *            デコ絵文字を表示させるTextView
     * @param position
     *            デコ絵文字一覧の要素位置(index)
     * @param width
     *            TextViewの横幅
     * @param height
     *            TextViewの縦幅
     * @return パラメータを設定したTextView
     */
    private TextView getCategoryTextView(TextView textView, int position,
            int width, int height, CharSequence charSequence) {
        textView.setBackgroundResource(R.drawable.buttons_background_type1);
        textView.setLayoutParams(new GridView.LayoutParams(width, height));
        textView.setPadding(0, 0, 0, 0);
        textView.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL);
        textView.setSingleLine();
        textView.setOnClickListener(mDecoEmojiOnClick);
        textView.setTag(mCurrentDecoEmojiList.get(position).getUri());

        textView.setText(charSequence);

        // 一覧表示される絵文字サイズを大きくする
        textView.setTextSize(DECOEMOJI_SIZE);

        addView(textView, true, Const.SWITCH_VALUE_DECO);

        return textView;
    }

    /**
     * デコ絵文字一覧を表示するTextViewのonClickイベントの設定を行う.
     */
    private final OnClickListener mDecoEmojiOnClick = new OnClickListener() {
        /**
         * デコ絵文字を表示しているTextView押下時のイベント.
         */
        public void onClick(View v) {
            if (mReturnValue.size() < Const.MAX_PREVIEW_TEXT) {
                DecoEmojiListResultInfo decoEmojiListResultInfo = new DecoEmojiListResultInfo();
                if (!(v instanceof TextView)) {
                    Log.e(Const.TAG, "mDecoEmojiOnClick.onClick() invalid view");
                    return;
                }
                TextView textView = (TextView) v;
                decoEmojiListResultInfo.setUri((String) textView.getTag());
                decoEmojiListResultInfo.setTag(Const.SWITCH_VALUE_DECO);
                mReturnValue.add(decoEmojiListResultInfo);

                if (Const.DEBUG) {
                    Log.d(Const.TAG,
                            "mDecoEmojiOnClick.onClick() Selected DecoEmoji. uri["
                                    + (String) textView.getTag() + "]");
                }

                // プレビューテキストに表示する
                if (mCurrentInputType == INPUT_TYPE_SINGLE) {
                    setResultData();
                } else {
                    // プレビューテキストに表示する
                    setPreviewText();
                }
            }
        }
    };

    /**
     * Unicode絵文字一覧を表示するTextViewのonClickイベントの設定を行う.
     */
    private OnClickListener mUnicodeEmojiOnClick = new OnClickListener() {

        /**
         * Unicode絵文字を表示しているTextView押下時のイベント.
         */
        public void onClick(View v) {
            if (mReturnValue.size() < Const.MAX_PREVIEW_TEXT) {
                DecoEmojiListResultInfo decoEmojiListResultInfo = new DecoEmojiListResultInfo();
                if (!(v instanceof TextView)) {
                    Log.e(Const.TAG,
                            "mUnicodeEmojiOnClick.onClick() invalid view");
                    return;
                }
                TextView textView = (TextView) v;
                decoEmojiListResultInfo.setCode((String) textView.getTag());
                decoEmojiListResultInfo.setTag(Const.SWITCH_VALUE_UNICODE);
                mReturnValue.add(decoEmojiListResultInfo);

                if (Const.DEBUG) {
                    Log.d(Const.TAG,
                            "mUnicodeEmojiOnClick.onClick() Selected UnicodeEmoji. code["
                                    + (String) textView.getTag() + "]");
                }

                // プレビューテキストに表示する
                if (mCurrentInputType == INPUT_TYPE_SINGLE) {
                    setResultData();
                } else {
                    // プレビューテキストに表示する
                    setPreviewText();
                }
            }
        }
    };

    /**
     * 表示しているデコ絵文字をクリアする.
     */
    private void clearDecoComponents() {
        // カテゴリ名称をクリアする
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(
                getApplicationContext(), R.layout.spinner_item);
        adapter.setDropDownViewResource(R.layout.select_dialog_singlechoice);
        adapter.add("");
        mCategoryName.setAdapter(adapter);

        // デコ絵文字一覧をクリアする
        mDecoEmojiListBaseLinear.removeAllViews();
        mCategoryInfoArray = null;
    }

    /**
     * Activityをfinish()する. 引数に応じて値を返却する。
     * 
     * @param isOnClickEnter
     *            true:確定ボタン押下後/false:確定ボタン押下後以外
     */
    private void sendResultData(boolean isOnClickEnter) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "sendResultData() start. isOnClickEnter["
                    + isOnClickEnter + "] isSendFinish[" + isSendFinish + "]");
        }

        // 入力履歴テーブル操作用DAOクラスを閉じる
        if (mInputHistoryDao != null) {
            mInputHistoryDao.closeHistories();
        }

        // 確定ボタン押下時のみ、Extraを送信する
        if (isOnClickEnter) {
            Intent data = (Intent) getIntent();
            data.putParcelableArrayListExtra(Const.EXTRA_RETURN_VALUE,
                    mReturnValue);
            setResult(DecoEmojiList.RESULT_OK, data);
        }

        // finish送信済みの時、finishを行わない
        if (!isSendFinish) {
            finish();
            isSendFinish = true;
        }

        // 遷移アニメを指定する
        overridePendingTransition(R.anim.decoemojilist_close_enter,
                R.anim.decoemojilist_close_exit);

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "sendResultData() end.");
        }
    }

    private void setResultData() {
        // 入力履歴をデータベースに保存する
        if (mReturnValue.size() != 0) {
            DecoEmojiListResultInfo decoEmojiListResultInfo;
            for (int i = 0; i < mReturnValue.size(); i++) {
                decoEmojiListResultInfo = mReturnValue.get(i);
                switch (decoEmojiListResultInfo.getTag()) {
                // デコ絵文字
                case Const.SWITCH_VALUE_DECO:
                    try {
                        mIDecoEmojiManagerIF
                                .aidl_changeHistory(decoEmojiListResultInfo
                                        .getUri());
                    } catch (RemoteException e) {
                        break;
                    }
                    break;

                // Unicode絵文字
                case Const.SWITCH_VALUE_UNICODE:
                    mInputHistoryDao
                            .updateHistoryUnicode(decoEmojiListResultInfo
                                    .getCode());
                    break;

                default:
                    throw new IllegalArgumentException("Unsupported tag "
                            + decoEmojiListResultInfo.getTag());
                }
            }
        }
        // 結果を返却する
        sendResultData(true);
    }

    /**
     * 一覧表示の行途中で改行が入る場合、最後の絵文字の背景を行の最後まで伸ばす。
     * 
     * @param viewCnt
     *            現在処理中の絵文字のViewTextインデックス
     * @param decoEmojiCounter
     *            現在処理中の絵文字のURIリストインデックス
     * @param position
     *            現在処理中の絵文字の行における位置
     * @param height
     *            行の高さ
     */
    private void updatePreViewLayOut(int viewCnt, int decoEmojiCounter,
            int position, int height) {
        int resizeWidth = (mListViewManager.getDecoEmojiMaxCols() - position)
                * mListViewManager.getDecoEmojiWidth()
                + (mListViewManager.getDisplayWidth() % mListViewManager
                        .getDecoEmojiMaxCols());
        TextView preView = (TextView) mDecoEmojiListInner
                .getChildAt(viewCnt - 1);
        mEmojiAssist.removeView(preView);
        mAddViewList.remove(preView);
        mDecoEmojiListInner.removeViewAt(viewCnt - 1);
        EmojiAssist.DecoEmojiTextInfo decoEmojiTextInfo = new EmojiAssist.DecoEmojiTextInfo();
        EmojiAssist.DecoEmojiTextInfo currentDecoEmojiInfo = mCurrentDecoEmojiList
                .get(decoEmojiCounter - 1);
        decoEmojiTextInfo.setUri(Const.PREFIX_URI
                + currentDecoEmojiInfo.getUri());
        decoEmojiTextInfo.setContext(getApplicationContext());
        decoEmojiTextInfo.setWidth(currentDecoEmojiInfo.getWidth());
        decoEmojiTextInfo.setHeight(currentDecoEmojiInfo.getHeight());
        decoEmojiTextInfo.setKind(currentDecoEmojiInfo.getKind());
        decoEmojiTextInfo.setPop(currentDecoEmojiInfo.getPop());
        decoEmojiTextInfo.setEmojiType(mEmojiType);
        CharSequence charSequence = mEmojiAssist
                .getDecoEmojiText(decoEmojiTextInfo);
        preView = getCategoryTextView(preView, decoEmojiCounter - 1,
                resizeWidth, height, charSequence);
        mDecoEmojiListInner.addView(preView);
    }

    /**
     * ページ情報保持クラス
     * 
     */
    static final class PageInfo {

        /** デコ絵文字表示件数 */
        private int mEmojiCount;
        /** デコ絵文字取得開始位置 */
        private int mEmojiStart;
        /** デコメピクチャ表示件数 */
        private int mPictureCount;
        /** デコメピクチャ開始位置 */
        private int mPictureStart;

        /**
         * コンストラクタ
         * 
         * @param emojiCount
         *            デコ絵文字表示件数
         * @param emojiStart
         *            デコ絵文字取得開始位置
         * @param pictureCount
         *            デコメピクチャ表示件数
         * @param pictureStart
         *            デコメピクチャ開始位置
         */
        public PageInfo(int emojiCount, int emojiStart, int pictureCount,
                int pictureStart) {
            mEmojiCount = emojiCount;
            mEmojiStart = emojiStart;
            mPictureCount = pictureCount;
            mPictureStart = pictureStart;
        }

        /**
         * デコ絵文字表示件数を設定する
         * 
         * @param count
         *            　デコ絵文字表示件数
         */
        public void setEmojiCount(int count) {
            mEmojiCount = count;
        }

        /**
         * デコ絵文字表示件数を取得する
         * 
         * @return デコ絵文字表示件数
         */
        public int getEmojiCount() {
            return mEmojiCount;
        }

        /**
         * デコ絵文字取得開始位置を設定する
         * 
         * @param start
         */
        public void setEmojiStart(int start) {
            mEmojiStart = start;
        }

        /**
         * デコ絵文字取得開始位置を取得する
         * 
         * @return
         */
        public int getEmojiStart() {
            return mEmojiStart;
        }

        /**
         * デコメピクチャ表示件数を設定する
         * 
         * @param count
         *            　デコメピクチャ表示件数
         */
        public void setPictureCount(int count) {
            mPictureCount = count;
        }

        /**
         * デコメピクチャ表示件数を取得する
         * 
         * @return デコメピクチャ表示件数
         */
        public int getPictureCount() {
            return mPictureCount;
        }

        /**
         * デコメピクチャ開始位置を設定する
         * 
         * @param start
         *            デコメピクチャ開始位置
         */
        public void setPictureStart(int start) {
            mPictureStart = start;
        }

        /**
         * デコメピクチャ開始位置を取得する。
         * 
         * @return デコメピクチャ開始位置
         */
        public int getPictureStart() {
            return mPictureStart;
        }

    }

    /**
     * リストにデコレーション絵文字が含まれている場合には、アニメーションの開始を行う.
     */
    private void startAnimation() {
        Boolean isDeco = false;
        if (mAddViewList != null && mAddViewList.size() > 0) {
            for (int i = 0; mAddViewList.size() > i; i++) {
                Spannable text = new SpannableString(mAddViewList.get(i)
                        .getText());
                int ret = mEmojiAssist.checkTextData(text);
                if (ret == EmojiAssist.TYPE_TEXT_DECOEMOJI) {
                    isDeco = true;
                    break;
                }
            }
            if (isDeco) {
                mEmojiAssist.startAnimation();
            } else {
                mEmojiAssist.stopAnimation();
            }
        } else {
            mEmojiAssist.stopAnimation();
        }
    }

    /**
     * 指定された種類のViewの追加を行う.
     * 
     * @param v
     *            追加するView
     * @param scale
     *            viewの幅に合わせた縮小設定
     * @param addViewKind
     *            追加するViewの種類
     */
    private void addView(TextView v, boolean scale, int addViewKind) {
        // 絵文字の描画をソフトウェアレンダリングに設定
        v.setLayerType(View.LAYER_TYPE_SOFTWARE, null);

        if ((addViewKind == Const.SWITCH_VALUE_DECO)
                || (addViewKind == Const.SWITCH_VALUE_BOTH)) {
            mEmojiAssist.addView(v, scale);
        }
        mAddViewList.add(v);
    }
}
