/***********************************************************************
 *    DecoEmojiListStarter.
 *
 * [File]
 *     DecoEmojiListStarter.java
 *
 *     Copyright (C) 2014 OMRON SOFTWARE Corporation.
 *     All rights reserved.
 *
 * [Description]
 *     Start for DecoEmojiList.
 ************************************************************************/
package com.emojisampleapplication;

import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.text.Editable;
import android.util.Log;
import android.view.Window;
import android.widget.EditText;
import android.widget.Toast;

import java.util.ArrayList;

import jp.co.omronsoft.android.emoji.EmojiAssist;

import com.emojisampleapplication.EmojiSampleApplication;

public class DecoEmojiListStarter extends Activity {

    /** For Debug Flg. */
    private static final Boolean DEBUG = false;

    /** For Debug Tag. */
    private static final String TAG = "DecoEmojiList";

    /** For Debug Class Name. */
    private static final String CLASS_NAME = "DecoEmojiListStarter#";

    /** Next Package Name. */
    public static final String NEXT_FULL_PACKAGE_NAME = "com.emojisampleapplication";

    /** Next Activity Name. */
    public static final String NEXT_FULL_ACTIVITY_NAME = "com.emojisampleapplication.DecoEmojiList";

    /** Switch Emoji Flg. */
    private static final String EXTRA_SWITCH_EMOJI_FLG = "switch_emoji_flg";

    /** Current Ime Height Portrait. */
    public static final String EXTRA_CURRENT_IME_HEIGHT_PORT = "current_ime_height_port";

    /** Current Ime Height Landscape. */
    public static final String EXTRA_CURRENT_IME_HEIGHT_LAND = "current_ime_height_land";

    /** Current Ime Height Portrait. */
    public static final String EXTRA_CURRENT_IME_ENABLE_DECO = "current_ime_enable_deco";

    /** Current Ime Height Landscape. */
    public static final String EXTRA_CURRENT_IME_ENABLE_CARRIER = "current_ime_enable_carrier";

    /** Current Input Type. */
    public static final String EXTRA_CURRENT_INPUT_TYPE = "current_input_type";

    /** Current Emoji Type. */
    public static final String EXTRA_EMOJI_TYPE = "emojiType";

    /** Return Value From DecoEmojiList. */
    private static final String EXTRA_RETURN_VALUE = "uri_list";

    /** Switch Flg [Both]. */
    private static final int SWITCH_VALUE_BOTH = 0;

    /** Switch Flg [Deco]. */
    private static final int SWITCH_VALUE_DECO = 1;

    /** Switch Flg [Carrier]. */
    private static final int SWITCH_VALUE_CARRIER = 2;

    /** Activity Not Found Exception. */
    private static final String EXCEPTION_ACTIVITY_NOT_FOUND = "ActivityNotFoundException";

    /** RuntimeException */
    private static final String EXCEPTION_RUNTIME = "RuntimeException";

    /** Request Code. */
    private static final int REQUEST_CODE = 0;

    /** Tag For DecoEmoji. */
    private static final int TAG_DECOEMOJI = 1;

    /**
     * @see android.app.Activity#onCreate
     */
    @Override
    public void onCreate(Bundle b) {
        if (DEBUG) {
            Log.d(TAG, CLASS_NAME + "onCreate() start.");
        }
        super.onCreate(b);

        // No Title Bar.
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        try {
            Intent intent = getIntent();
            int currentImeHeightPort = intent.getIntExtra(
                    EXTRA_CURRENT_IME_HEIGHT_PORT, 0);
            int currentImeHeightLand = intent.getIntExtra(
                    EXTRA_CURRENT_IME_HEIGHT_LAND, 0);
            boolean currentImeDeco = intent.getBooleanExtra(
                    EXTRA_CURRENT_IME_ENABLE_DECO, false);
            boolean currentImeCarrier = intent.getBooleanExtra(
                    EXTRA_CURRENT_IME_ENABLE_CARRIER, false);
            int currentInputType = intent.getIntExtra(
                    EXTRA_CURRENT_INPUT_TYPE, 0);
            int emojiType = intent.getIntExtra(
                    EXTRA_EMOJI_TYPE, 0);

            if (DEBUG) {
                Log.d(TAG, CLASS_NAME + "onCreate() SWITCH_EMOJI_FLG["
                        + SWITCH_VALUE_BOTH + "]" + " CURRENT_IME_HEIGHT_PORT["
                        + currentImeHeightPort + "]"
                        + " CURRENT_IME_HEIGHT_LAND[" + currentImeHeightLand
                        + "]");
            }

            intent = new Intent();
            intent.setClassName(this, NEXT_FULL_ACTIVITY_NAME);
            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
            if ((currentImeDeco == true) && (currentImeCarrier == true)) {
                intent.putExtra(EXTRA_SWITCH_EMOJI_FLG, SWITCH_VALUE_BOTH);
            } else if ((currentImeDeco == true) && (currentImeCarrier == false)) {
                intent.putExtra(EXTRA_SWITCH_EMOJI_FLG, SWITCH_VALUE_DECO);
            } else {
                intent.putExtra(EXTRA_SWITCH_EMOJI_FLG, SWITCH_VALUE_CARRIER);
            }
            intent.putExtra(EXTRA_CURRENT_IME_HEIGHT_PORT, currentImeHeightPort);
            intent.putExtra(EXTRA_CURRENT_IME_HEIGHT_LAND, currentImeHeightLand);
            intent.putExtra(EXTRA_CURRENT_INPUT_TYPE, currentInputType);
            intent.putExtra(EXTRA_EMOJI_TYPE, emojiType);
            startActivityForResult(intent, REQUEST_CODE);

            // Set Animation
            overridePendingTransition(R.anim.decoemojilist_open_enter,
                    R.anim.decoemojilist_open_exit);

        } catch (ActivityNotFoundException e) {
            Toast.makeText(getApplicationContext(),
                    EXCEPTION_ACTIVITY_NOT_FOUND, Toast.LENGTH_SHORT).show();
            e.printStackTrace();
        } catch (RuntimeException e) {
            Toast.makeText(getApplicationContext(), EXCEPTION_RUNTIME,
                    Toast.LENGTH_SHORT).show();
            e.printStackTrace();
        }
        if (DEBUG) {
            Log.d(TAG, CLASS_NAME + "onCreate() end.");
        }
    }

    /**
     * @see android.app.Activity#onActivityResult
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (DEBUG) {
            Log.d(TAG, CLASS_NAME + "onActivityResult() start.");
        }

        if (requestCode == REQUEST_CODE && resultCode == RESULT_OK) {
            ArrayList<DecoEmojiListResultInfo> result = data
                    .getParcelableArrayListExtra(EXTRA_RETURN_VALUE);
            commitDecoEmojiListResultString(result);
        }
        finish();

        if (DEBUG) {
            Log.d(TAG, CLASS_NAME + "onActivityResult() end.");
        }
    }

    /**
     * @see android.app.Activity#onConfigurationChanged
     */
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        if (DEBUG) {
            Log.d(TAG, CLASS_NAME + "onConfigurationChanged() start.");
        }
        super.onConfigurationChanged(newConfig);
        if (DEBUG) {
            Log.d(TAG, CLASS_NAME + "onConfigurationChanged() end.");
        }
    };

    private void commitDecoEmojiListResultString(
            ArrayList<DecoEmojiListResultInfo> result) {
        if (result != null) {
            for (int i = 0; i < result.size(); i++) {

                DecoEmojiListResultInfo resultInfo = result.get(i);
                CharSequence charSequence = null;
                EmojiAssist assist = EmojiAssist.getInstance();
                if (resultInfo.getTag() == TAG_DECOEMOJI) {
                    // デコ絵文字
                    charSequence = assist.getDecoEmojiText(
                            "file://" + resultInfo.getUri(), this);

                } else {
                    // Unicode絵文字
                    charSequence = resultInfo.getCode();
                }

                if (charSequence != null) {
                    // カーソル位置に絵文字を挿入
                    EmojiSampleApplication obj = EmojiSampleApplication.getInstance();
                    if (obj == null) {
                        return;
                    }

                    EditText text = obj.getEditText();
                    if (text == null) {
                        return;
                    }

                    int start = text.getSelectionStart();
                    int end = text.getSelectionEnd();

                    if (start < 0) {
                        start = 0;
                    }
                    if (end < 0) {
                        end = 0;
                    }

                    assist.addView(text);
                    Editable editable = text.getText();
                    editable.replace(Math.min(start, end), Math.max(start, end), charSequence);
                    assist.setPictureScale(false);
                }
            }
        }
    }
}
