/***********************************************************************
 *    SampleApplication
 *
 * [File]
 *     SampleApplication.java
 *
 *     Copyright (C) 2014 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     Sample Application Activity Class.
 ************************************************************************/
package com.emojisampleapplication;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Point;
import android.os.Bundle;
import android.text.SpannableStringBuilder;
import android.view.Display;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.emojisampleapplication.DecoEmojiListStarter;

import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.IDecoEmojiConstant;
import jp.co.omronsoft.android.emoji.EmojiAssist;
import jp.co.omronsoft.android.text.style.DecoEmojiSpan;


public class EmojiSampleApplication extends Activity {
    private static final int EMOJILIST_INPUT_TYPE_MULTI = 0;
    private static final int EMOJILIST_INPUT_TYPE_SINGLE = 1;

    private static final int MENU_ID_INPUT_TYPE_MULTI = Menu.FIRST;
    private static final int MENU_ID_INPUT_TYPE_SINGLE = (Menu.FIRST + 1);

    private static final String FILE_CHECK_PATH = "/mnt/sdcard/Download/";

    private EmojiAssist mAssist;
    private EditText mEditText;
    private TextView mTextView;

    private boolean mIsToggleButton1 = true;
    private boolean mIsToggleButton2 = true;

    private int mIsCheckBox1 = EmojiAssist.EMOJITYPE_PICTD_SQ20;
    private int mIsCheckBox2 = EmojiAssist.EMOJITYPE_PICTD_SQNOT20;
    private int mIsCheckBox3 = EmojiAssist.EMOJITYPE_PICTD_RECT;
    private int mIsCheckBox4 = EmojiAssist.EMOJITYPE_PICTURE;

    /** Created instance. */
    private static EmojiSampleApplication sSelf = null;

    /**
     * Get edit text.
     * @return The edit text
     */
    public EditText getEditText() {
        return mEditText;
    }

    /**
     * Get instance.
     * @return The created instance
     */
    public static EmojiSampleApplication getInstance() {
        return sSelf;
    }

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        sSelf = this;
        setContentView(R.layout.main);

        ToggleButton tb1 = (ToggleButton) findViewById(R.id.toggleButton1);
        tb1.setChecked(true);
        tb1.setOnCheckedChangeListener(new OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                mIsToggleButton1 = isChecked;
                makeEmojiText();
                makeEmojiTextView();
            }
        });

        ToggleButton tb2 = (ToggleButton) findViewById(R.id.toggleButton2);
        tb2.setChecked(true);
        tb2.setOnCheckedChangeListener(new OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                mIsToggleButton2 = isChecked;
                makeEmojiText();
            }
        });

        // Span情報の取得
        Button spanButton = (Button) findViewById(R.id.spanButton);
        spanButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                // 入力された絵文字から情報を取得
                CharSequence charSeq = mEditText.getText();
                SpannableStringBuilder ssb = SpannableStringBuilder.valueOf(charSeq);
                DecoEmojiSpan[] decoEmojiSpans = ssb.getSpans(0, ssb.length(), DecoEmojiSpan.class);

                // 1文字目のDecoEmojiSpanから各情報を取得
                if (decoEmojiSpans.length != 0) {
                    String uri = decoEmojiSpans[0].getURI();
                    int width = decoEmojiSpans[0].getWidth();
                    int height = decoEmojiSpans[0].getHeight();
                    int pop = decoEmojiSpans[0].getPopFlag();
                    boolean ret = decoEmojiSpans[0].isTypePicture();
                    // 取得した各情報を表示
                    TextView info = (TextView) findViewById(R.id.spanText);
                    info.setText("Span情報\n" +
                                 "URI : " + uri + "\n" +
                                 "EmojiType : " + ret + "\n" +
                                 "Width : " + width + "\t" + "Height : " + height + "\n" +
                                 "Pop : " + pop+ "\n");
                }
            }
        });

        // ファイル差分チェック実施要求(/mnt/sdacrd/Download/配下)
        Button fileCheckButton = (Button) findViewById(R.id.fileCheckButton);
        fileCheckButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                // パッケージ名取得
                String packageName = getPackageName();
                // Intentのインスタンス生成
                Intent intent = new Intent(IDecoEmojiConstant.ACTION_FILE_CHECK);
                // Bundleに情報格納
                Bundle bundle = new Bundle();
                bundle.putString(IDecoEmojiConstant.BROADCAST_PACKAGE_NAME_TAG, packageName);
                bundle.putString(IDecoEmojiConstant.BROADCAST_PATH_TAG, FILE_CHECK_PATH);
                // IntentにBundle格納
                intent.putExtras(bundle);
                // Intent送信
                sendBroadcast(intent);
            }
        });

        CheckBox checkBox1 = (CheckBox) findViewById(R.id.checkBox1);
        CheckBox checkBox2 = (CheckBox) findViewById(R.id.checkBox2);
        CheckBox checkBox3 = (CheckBox) findViewById(R.id.checkBox3);
        CheckBox checkBox4 = (CheckBox) findViewById(R.id.checkBox4);

        checkBox1.setChecked(true);
        checkBox2.setChecked(true);
        checkBox3.setChecked(true);
        checkBox4.setChecked(true);

        checkBox1.setOnClickListener(new View.OnClickListener() {
            @Override
            // チェックボックスがクリックされた時に呼び出されます
            public void onClick(View v) {
                CheckBox checkBox = (CheckBox) v;
                // チェックボックスのチェック状態を取得します
                boolean checked = checkBox.isChecked();
                mIsCheckBox1 = 0;
                if (checked) {
                    // 絵文字種別 絵文字D(20×20)を表示対象とする
                    mIsCheckBox1 = EmojiAssist.EMOJITYPE_PICTD_SQ20;
                }
                makeEmojiText();
            }
        });

        checkBox2.setOnClickListener(new View.OnClickListener() {
            @Override
            // チェックボックスがクリックされた時に呼び出されます
            public void onClick(View v) {
                CheckBox checkBox = (CheckBox) v;
                // チェックボックスのチェック状態を取得します
                boolean checked = checkBox.isChecked();
                mIsCheckBox2 = 0;
                if (checked) {
                    // 絵文字種別 絵文字D(正方形)を表示対象とする
                    mIsCheckBox2 = EmojiAssist.EMOJITYPE_PICTD_SQNOT20;
                }
                makeEmojiText();
            }
        });

        checkBox3.setOnClickListener(new View.OnClickListener() {
            @Override
            // チェックボックスがクリックされた時に呼び出されます
            public void onClick(View v) {
                CheckBox checkBox = (CheckBox) v;
                // チェックボックスのチェック状態を取得します
                boolean checked = checkBox.isChecked();
                mIsCheckBox3 = 0;
                if (checked) {
                    // 絵文字種別 絵文字D(長方形)を表示対象とする
                    mIsCheckBox3 = EmojiAssist.EMOJITYPE_PICTD_RECT;
                }
                makeEmojiText();
            }
        });

        checkBox4.setOnClickListener(new View.OnClickListener() {
            @Override
            // チェックボックスがクリックされた時に呼び出されます
            public void onClick(View v) {
                CheckBox checkBox = (CheckBox) v;
                // チェックボックスのチェック状態を取得します
                boolean checked = checkBox.isChecked();
                mIsCheckBox4 = 0;
                if (checked) {
                    // 絵文字種別 デコメピクチャを表示対象とする
                    mIsCheckBox4 = EmojiAssist.EMOJITYPE_PICTURE;
                }
                makeEmojiText();
            }
        });

        Button button = (Button) findViewById(R.id.viewButton);
        button.setOnClickListener(new View.OnClickListener() {
            @Override
            // EditTextに入力した文字列をTextViewに表示
            public void onClick(View v) {
                makeEmojiTextView();
                mTextView.setText(mEditText.getText());
            }
        });

        mAssist = EmojiAssist.getInstance();
        TextView version = (TextView) findViewById(R.id.textVersion);
        version.setText("EmojiPKG version : " + mAssist.getVersion(this));
        makeEmojiText();
    }

    @Override
    protected void onPause() {
        super.onPause();
        // EmojiAssistのアニメ動作を停止させる
        mAssist.stopAnimation();
    }

    @Override
    protected void onResume() {
        super.onResume();
        // EmojiAssistのアニメ動作を開始させる
        mAssist.startAnimation();
    }

    private void makeEmojiText() {
        mEditText = (EditText) findViewById(R.id.editText);
        mEditText.requestFocus();
        Bundle bundle = mEditText.getInputExtras(true);

        if (mIsToggleButton1 == true) {
            bundle.putBoolean("allowEmoji", true);
        } else {
            bundle.putBoolean("allowEmoji", false);
        }
        if (mIsToggleButton2 == true) {
            bundle.putBoolean("allowDecoEmoji", true);
        } else {
            bundle.putBoolean("allowDecoEmoji", false);
        }

        int emojiType = 0;
        // チェックボックスで指定した絵文字種別を表示対象とする
        emojiType = mIsCheckBox1 | mIsCheckBox2 | mIsCheckBox3 | mIsCheckBox4;
        bundle.putInt("emojiType", emojiType);
        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        inputMethodManager.restartInput(this.getCurrentFocus());

        if (mIsToggleButton2 == true) {
            // デコメ絵文字の描画をソフトウェアレンダリングに設定
            mEditText.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
            // EmojiAssistに取り込み、絵文字表示を可能にする
            // TextViewを EmojiAssist に割り当てる(別で実施)
            mAssist.addView(mEditText, true);
            // popの識別子を表示させる
            mAssist.setDisplayPop(true);
        }
    }

    private void makeEmojiTextView() {
        mTextView = (TextView) findViewById(R.id.serviceText);
        mTextView.requestFocus();
        Bundle bundle = mTextView.getInputExtras(true);

        // デコメ絵文字のクリックを可能にする
        bundle.putBoolean("tagsSensible", true);

        // デコメ絵文字の描画をソフトウェアレンダリングに設定
        mTextView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
        // EmojiAssistに取り込み、絵文字表示を可能にする
        // TextViewを EmojiAssist に割り当てる(別で実施)
        mAssist.addView(mTextView, true);
        // popの識別子を表示させる
        mAssist.setDisplayPop(true);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.clear();
        // デコ絵文字、キャリア絵文字のどちらかが有効であればメニューを表示
        menu.add(Menu.NONE, MENU_ID_INPUT_TYPE_MULTI, Menu.NONE,
                R.string.menu_list_multi_txt);
        menu.add(Menu.NONE, MENU_ID_INPUT_TYPE_SINGLE, Menu.NONE,
                R.string.menu_list_single_txt);
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        menu.clear();
        // デコ絵文字、キャリア絵文字のどちらかが有効であればメニューを表示
        menu.add(Menu.NONE, MENU_ID_INPUT_TYPE_MULTI, Menu.NONE,
                R.string.menu_list_multi_txt);
        menu.add(Menu.NONE, MENU_ID_INPUT_TYPE_SINGLE, Menu.NONE,
                R.string.menu_list_single_txt);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        boolean ret = false;
        switch (item.getItemId()) {
        case MENU_ID_INPUT_TYPE_MULTI:
            // 連続モード
            startEmojiList(EMOJILIST_INPUT_TYPE_MULTI);
            ret = true;
            break;
        case MENU_ID_INPUT_TYPE_SINGLE:
            // シングルモード
            startEmojiList(EMOJILIST_INPUT_TYPE_SINGLE);
            ret = true;
            break;
        default:
            break;
        }
        return ret;
    }

    private void startEmojiList(int inputType) {
        // DecoEmojiList起動用のactivityを起動
        Intent intent = new Intent();
        intent.setClass(this, DecoEmojiListStarter.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

        Display display = getWindowManager().getDefaultDisplay();
        Point p = new Point();
        display.getSize(p);
        Configuration config = getResources().getConfiguration();
        int currentImeHeight = p.y;
        if (config.orientation == Configuration.ORIENTATION_LANDSCAPE) {
            currentImeHeight = p.x;
        }

        intent.putExtra(DecoEmojiListStarter.EXTRA_CURRENT_IME_HEIGHT_PORT,
                currentImeHeight / 2);
        intent.putExtra(DecoEmojiListStarter.EXTRA_CURRENT_IME_HEIGHT_LAND,
                currentImeHeight / 2);
        intent.putExtra(DecoEmojiListStarter.EXTRA_CURRENT_IME_ENABLE_CARRIER,
                mIsToggleButton1);
        intent.putExtra(DecoEmojiListStarter.EXTRA_CURRENT_IME_ENABLE_DECO,
                mIsToggleButton2);
        intent.putExtra(DecoEmojiListStarter.EXTRA_CURRENT_INPUT_TYPE,
                inputType);
        int emojiType = mIsCheckBox1 | mIsCheckBox2 | mIsCheckBox3 | mIsCheckBox4;
        intent.putExtra(DecoEmojiListStarter.EXTRA_EMOJI_TYPE, emojiType);
        this.startActivity(intent);
    }
}
