/***********************************************************************
 *    SampleEmojiReceiver
 *
 * [File]
 *     SampleEmojiReceiver.java
 *
 *     Copyright (C) 2014 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     Sample Emoji Broadcast Receiver Class.
 ************************************************************************/
package com.emojisampleapplication;

import java.util.List;

import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiAttrInfo;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.IDecoEmojiConstant;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

public class SampleEmojiReceiver extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {

        // 受信すべき更新通知（Broadcast）
        // intent識別子を取得
        String action = intent.getAction();

        // 1.デコレーション絵文字情報更新通知
        if (action.equals(IDecoEmojiConstant.ACTION_CHANGE_DATABASE)) {
            // IntentBundleを取得する
            Bundle bundle = intent.getExtras();
            // Typeを取得する。
            int type = bundle.getInt(IDecoEmojiConstant.BROADCAST_TYPE_TAG);
            if (type != IDecoEmojiConstant.FLAG_SET_TO_INITIALIZING) {
                // URIを取得する。
                String uri = bundle.getString(IDecoEmojiConstant.BROADCAST_URI_TAG);
                // 更新時のTimeStampを取得する。
                String timestamp = bundle.getString(IDecoEmojiConstant.BROADCAST_TIMESTAMP_TAG);
            }
            if (type == IDecoEmojiConstant.FLAG_INSERT) {
                // 更新区分 追加時の処理
            } else if (type == IDecoEmojiConstant.FLAG_DELETE) {
                // 更新区分 削除時の処理
            } else if (type == IDecoEmojiConstant.FLAG_SET_TO_INITIALIZING) {
                // 更新区分 初期化時の処理
            }
        }

        // 2.デコレーション絵文字辞書更新通知
        if (action.equals(IDecoEmojiConstant.ACTION_DECOEMOJI_RESULT)){
            Bundle bundle = intent.getExtras();
            int type = bundle.getInt(IDecoEmojiConstant.BROADCAST_TYPE_TAG);
            if (type == IDecoEmojiConstant.FLAG_SET_TO_INITIALIZING) {
                // DB初期化中の処理を行う.
            } else {
                List<DecoEmojiAttrInfo> receivedatalist = bundle.getParcelableArrayList(IDecoEmojiConstant.BROADCAST_DATA_TAG);

                // 受信データが empty か nullでなければ, 更新処理を行う.
                if (receivedatalist != null && !receivedatalist.isEmpty()) {
                    // Get the number of the included DecoEmoji information.
                    int len = receivedatalist.size();

                    // Generate an array of DecoEmojiAttrInfo.
                    DecoEmojiAttrInfo[] decoemojiattrinfo = new DecoEmojiAttrInfo[len];
                    receivedatalist.toArray(decoemojiattrinfo);

                    // getData
                    for ( int i = 0; i < len; i++) {
                        //get Id
                        int decoEmojiId = decoemojiattrinfo[i].getId();
                        for ( int j = 0; j < 10; j++) {    // 絵文字１つにつき最大10件の情報
                            //get Name
                            String name = decoemojiattrinfo[i].getName(j);
                            //get Part
                            byte part = decoemojiattrinfo[i].getPart(j);
                            //get Note
                            String note = decoemojiattrinfo[i].getNote(j);

                            // Nameがnullか空文字でなければ, DecoEmoji辞書を更新する.
                        }
                    }
                }
            }
        }

        // 3.絵文字マネージャー状態変更通知
        if (action.equals(IDecoEmojiConstant.ACTION_STATE_CHANGED)) {
            // IntentBundleを取得する
            Bundle bundle = intent.getExtras();
            // 状態変更区分を取得する
            int state = bundle.getInt(IDecoEmojiConstant.BROADCAST_STATE_TAG);
            if (state == IDecoEmojiConstant.STATE_INIT_COMPLETE) {
                // 初期化完了時の処理を行う
            } else if (state == IDecoEmojiConstant.STATE_FILE_CHECK_COMPLETE) {
                // ファイルチェック完了時の処理を行う
            }
        }

        // 4.絵文字マネージャー状態通知
        if (action.equals("com.emojisampleapplication.MANAGER_STATE")) {
            // IntentBundleを取得する
            Bundle bundle = intent.getExtras();
            // 状態区分を取得する
            int state = bundle.getInt(IDecoEmojiConstant.BROADCAST_MANAGER_STATE_TAG);
            if (state == IDecoEmojiConstant.INITIALIZING) {
                // 初期化中状態受信時の処理を行う
            } else if (state == IDecoEmojiConstant.NG_FILE_CHECK_PATH) {
                // 指定フォルダ不正時の処理を行う
            }
        }
    }
}
